"""
This type stub file was generated by pyright.
"""

import sys
from dataclasses import dataclass
from typing import Any, Dict, Optional, Union
from typing_extensions import TypeAlias
from IPython.testing.skipdoctest import skip_doctest
from IPython.utils.coloransi import TermColors
from IPython.utils.colorable import Colorable
from IPython.utils.decorators import undoc

"""Tools for inspecting Python objects.

Uses syntax highlighting for presenting the various information elements.

Similar in spirit to the inspect module, but all calls take a name argument to
reference the name under which an object is being read.
"""
__all__ = ['Inspector', 'InspectColors']
if sys.version_info <= (3, 10):
    ...
else:
    ...
HOOK_NAME = ...
UnformattedBundle: TypeAlias = ...
Bundle: TypeAlias = ...
@dataclass
class OInfo:
    ismagic: bool
    isalias: bool
    found: bool
    namespace: Optional[str]
    parent: Any
    obj: Any
    def get(self, field):
        """Get a field from the object for backward compatibility with before 8.12

        see https://github.com/h5py/h5py/issues/2253
        """
        ...
    


def pylight(code): # -> None:
    ...

_func_call_docstring = ...
_object_init_docstring = ...
_builtin_type_docstrings = ...
_builtin_func_type = ...
_builtin_meth_type = ...
Colors = TermColors
InspectColors = ...
info_fields = ...
def object_info(**kw):
    """Make an object info dict with all fields present."""
    ...

def get_encoding(obj): # -> None:
    """Get encoding for python source file defining obj

    Returns None if obj is not defined in a sourcefile.
    """
    ...

def getdoc(obj) -> Union[str, None]:
    """Stable wrapper around inspect.getdoc.

    This can't crash because of attribute problems.

    It also attempts to call a getdoc() method on the given object.  This
    allows objects which provide their docstrings via non-standard mechanisms
    (like Pyro proxies) to still be inspected by ipython's ? system.
    """
    ...

def getsource(obj, oname=...) -> Union[str, None]:
    """Wrapper around inspect.getsource.

    This can be modified by other projects to provide customized source
    extraction.

    Parameters
    ----------
    obj : object
        an object whose source code we will attempt to extract
    oname : str
        (optional) a name under which the object is known

    Returns
    -------
    src : unicode or None

    """
    ...

def is_simple_callable(obj):
    """True if obj is a function ()"""
    ...

@undoc
def getargspec(obj):
    """Wrapper around :func:`inspect.getfullargspec`

    In addition to functions and methods, this can also handle objects with a
    ``__call__`` attribute.

    DEPRECATED: Deprecated since 7.10. Do not use, will be removed.
    """
    ...

@undoc
def format_argspec(argspec):
    """Format argspect, convenience wrapper around inspect's.

    This takes a dict instead of ordered arguments and calls
    inspect.format_argspec with the arguments in the necessary order.

    DEPRECATED (since 7.10): Do not use; will be removed in future versions.
    """
    ...

@undoc
def call_tip(oinfo, format_call=...):
    """DEPRECATED since 6.0. Extract call tip data from an oinfo dict."""
    ...

def find_file(obj) -> str:
    """Find the absolute path to the file where an object was defined.

    This is essentially a robust wrapper around `inspect.getabsfile`.

    Returns None if no file can be found.

    Parameters
    ----------
    obj : any Python object

    Returns
    -------
    fname : str
        The absolute path to the file where the object was defined.
    """
    ...

def find_source_lines(obj): # -> None:
    """Find the line number in a file where an object was defined.

    This is essentially a robust wrapper around `inspect.getsourcelines`.

    Returns None if no file can be found.

    Parameters
    ----------
    obj : any Python object

    Returns
    -------
    lineno : int
        The line number where the object definition starts.
    """
    ...

class Inspector(Colorable):
    def __init__(self, color_table=..., code_color_table=..., scheme=..., str_detail_level=..., parent=..., config=...) -> None:
        ...
    
    def set_active_scheme(self, scheme): # -> None:
        ...
    
    def noinfo(self, msg, oname): # -> None:
        """Generic message when no information is found."""
        ...
    
    def pdef(self, obj, oname=...): # -> None:
        """Print the call signature for any callable object.

        If the object is a class, print the constructor information."""
        ...
    
    @skip_doctest
    def pdoc(self, obj, oname=..., formatter=...): # -> None:
        """Print the docstring for any object.

        Optional:
        -formatter: a function to run the docstring through for specially
        formatted docstrings.

        Examples
        --------
        In [1]: class NoInit:
           ...:     pass

        In [2]: class NoDoc:
           ...:     def __init__(self):
           ...:         pass

        In [3]: %pdoc NoDoc
        No documentation found for NoDoc

        In [4]: %pdoc NoInit
        No documentation found for NoInit

        In [5]: obj = NoInit()

        In [6]: %pdoc obj
        No documentation found for obj

        In [5]: obj2 = NoDoc()

        In [6]: %pdoc obj2
        No documentation found for obj2
        """
        ...
    
    def psource(self, obj, oname=...): # -> None:
        """Print the source code for an object."""
        ...
    
    def pfile(self, obj, oname=...): # -> None:
        """Show the whole file where an object was defined."""
        ...
    
    def format_mime(self, bundle: UnformattedBundle) -> Bundle:
        """Format a mimebundle being created by _make_info_unformatted into a real mimebundle"""
        ...
    
    def pinfo(self, obj, oname=..., formatter=..., info: Optional[OInfo] = ..., detail_level=..., enable_html_pager=..., omit_sections=...): # -> None:
        """Show detailed information about an object.

        Optional arguments:

        - oname: name of the variable pointing to the object.

        - formatter: callable (optional)
              A special formatter for docstrings.

              The formatter is a callable that takes a string as an input
              and returns either a formatted string or a mime type bundle
              in the form of a dictionary.

              Although the support of custom formatter returning a string
              instead of a mime type bundle is deprecated.

        - info: a structure with some information fields which may have been
          precomputed already.

        - detail_level: if set to 1, more information is given.

        - omit_sections: set of section keys and titles to omit
        """
        ...
    
    def info(self, obj, oname=..., info=..., detail_level=...) -> Dict[str, Any]:
        """Compute a dict with detailed information about an object.

        Parameters
        ----------
        obj : any
            An object to find information about
        oname : str (default: '')
            Name of the variable pointing to `obj`.
        info : (default: None)
            A struct (dict like with attr access) with some information fields
            which may have been precomputed already.
        detail_level : int (default:0)
            If set to 1, more information is given.

        Returns
        -------
        An object info dict with known fields from `info_fields`. Keys are
        strings, values are string or None.
        """
        ...
    
    def psearch(self, pattern, ns_table, ns_search=..., ignore_case=..., show_all=..., *, list_types=...): # -> None:
        """Search namespaces with wildcards for objects.

        Arguments:

        - pattern: string containing shell-like wildcards to use in namespace
          searches and optionally a type specification to narrow the search to
          objects of that type.

        - ns_table: dict of name->namespaces for search.

        Optional arguments:

          - ns_search: list of namespace names to include in search.

          - ignore_case(False): make the search case-insensitive.

          - show_all(False): show all names, including those starting with
            underscores.

          - list_types(False): list all available object types for object matching.
        """
        ...
    





from typing import Any, Callable

from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from collections import OrderedDict
import inspect
import traceback
import weakref

from .logs import logger, _handle_exception

class Event(object):
    def __init__(self, type: str, native=None, **kwargs): ...
    @property
    def source(self): ...
    @property
    def sources(self): ...
    def _push_source(self, source): ...
    def _pop_source(self): ...
    @property
    def type(self): ...
    @property
    def native(self): ...
    @property
    def handled(self): ...
    @handled.setter
    def handled(self, val): ...
    @property
    def blocked(self): ...
    @blocked.setter
    def blocked(self, val): ...
    def __repr__(self): ...
    def __str__(self): ...

_event_repr_depth: int = ...

class EventEmitter(object):
    def __init__(self, source: Any = None, type: str | None = None, event_class: Event = ...): ...
    @property
    def ignore_callback_errors(self): ...
    @ignore_callback_errors.setter
    def ignore_callback_errors(self, val): ...
    @property
    def print_callback_errors(self): ...
    @print_callback_errors.setter
    def print_callback_errors(self, val): ...
    @property
    def callback_refs(self): ...
    @property
    def callbacks(self): ...
    @property
    def source(self): ...
    @source.setter
    def source(self, s): ...
    def connect(
        self,
        callback: Callable | tuple,
        ref: str | bool = False,
        position: str = "first",
        before: Callable | str | ArrayLike | None = None,
        after: Callable | str | ArrayLike | None = None,
    ): ...
    def disconnect(self, callback=None): ...
    def _normalize_cb(self, callback): ...
    def __call__(self, *args, **kwargs): ...
    def _invoke_callback(self, cb, event): ...
    def _prepare_event(self, *args, **kwargs): ...
    def blocked(self, callback=None): ...
    def block(self, callback=None): ...
    def unblock(self, callback=None): ...
    def blocker(self, callback=None): ...

class WarningEmitter(EventEmitter):
    def __init__(self, message, *args, **kwargs): ...
    def connect(self, cb, *args, **kwargs): ...
    def _invoke_callback(self, cb, event): ...
    def _warn(self, cb): ...

class EmitterGroup(EventEmitter):
    def __init__(self, source: Any = None, auto_connect: bool = True, **emitters): ...
    def __getitem__(self, name): ...
    def __setitem__(self, name, emitter): ...
    def add(self, auto_connect=None, **kwargs): ...
    @property
    def emitters(self): ...
    def __iter__(self): ...
    def block_all(self): ...
    def unblock_all(self): ...
    def connect(self, callback, ref=False, position="first", before=None, after=None): ...
    def disconnect(self, callback=None): ...
    def _connect_emitters(self, connect): ...
    @property
    def ignore_callback_errors(self): ...
    @ignore_callback_errors.setter
    def ignore_callback_errors(self, ignore): ...

class EventBlocker(object):
    def __init__(self, target, callback=None): ...
    def __enter__(self): ...
    def __exit__(self, *args): ...



from numpy.random import RandomState
from tempfile import TemporaryDirectory as TemporaryDirectory
from os.path import exists as exists, join as join
from ._base import RemoteFileMetadata as RemoteFileMetadata, load_descr as load_descr
from numpy import ndarray
from ..utils._bunch import Bunch
from gzip import GzipFile as GzipFile
from .._typing import Int
from ..utils import check_random_state as check_random_state
from . import get_data_home as get_data_home
import logging
import os

import numpy as np
import joblib


# The original data can be found in:
# https://archive.ics.uci.edu/ml/machine-learning-databases/covtype/covtype.data.gz
ARCHIVE = ...

logger = ...

# Column names reference:
# https://archive.ics.uci.edu/ml/machine-learning-databases/covtype/covtype.info
FEATURE_NAMES: list = ...
TARGET_NAMES: list = ...


def fetch_covtype(
    *,
    data_home: None | str = None,
    download_if_missing: bool = True,
    random_state: RandomState | None | Int = None,
    shuffle: bool = False,
    return_X_y: bool = False,
    as_frame: bool = False,
) -> Bunch | tuple[Bunch, tuple] | tuple[ndarray, ndarray]:
    ...



"""
This type stub file was generated by pyright.
"""

from IPython.core import magic_arguments
from IPython.core.magic import Magics, cell_magic, line_cell_magic, line_magic, magics_class
from IPython.testing.skipdoctest import skip_doctest

"""Implementation of magic functions for interaction with the OS.

Note: this module is named 'osm' instead of 'os' to avoid a collision with the
builtin.
"""
@magics_class
class OSMagics(Magics):
    """Magics to interact with the underlying OS (shell-type functionality).
    """
    cd_force_quiet = ...
    def __init__(self, shell=..., **kwargs) -> None:
        ...
    
    def isexec(self, file):
        """
        Test for executable file on non POSIX system
        """
        ...
    
    @skip_doctest
    @line_magic
    def alias(self, parameter_s=...): # -> None:
        """Define an alias for a system command.

        '%alias alias_name cmd' defines 'alias_name' as an alias for 'cmd'

        Then, typing 'alias_name params' will execute the system command 'cmd
        params' (from your underlying operating system).

        Aliases have lower precedence than magic functions and Python normal
        variables, so if 'foo' is both a Python variable and an alias, the
        alias can not be executed until 'del foo' removes the Python variable.

        You can use the %l specifier in an alias definition to represent the
        whole line when the alias is called.  For example::

          In [2]: alias bracket echo "Input in brackets: <%l>"
          In [3]: bracket hello world
          Input in brackets: <hello world>

        You can also define aliases with parameters using %s specifiers (one
        per parameter)::

          In [1]: alias parts echo first %s second %s
          In [2]: %parts A B
          first A second B
          In [3]: %parts A
          Incorrect number of arguments: 2 expected.
          parts is an alias to: 'echo first %s second %s'

        Note that %l and %s are mutually exclusive.  You can only use one or
        the other in your aliases.

        Aliases expand Python variables just like system calls using ! or !!
        do: all expressions prefixed with '$' get expanded.  For details of
        the semantic rules, see PEP-215:
        https://peps.python.org/pep-0215/.  This is the library used by
        IPython for variable expansion.  If you want to access a true shell
        variable, an extra $ is necessary to prevent its expansion by
        IPython::

          In [6]: alias show echo
          In [7]: PATH='A Python string'
          In [8]: show $PATH
          A Python string
          In [9]: show $$PATH
          /usr/local/lf9560/bin:/usr/local/intel/compiler70/ia32/bin:...

        You can use the alias facility to access all of $PATH.  See the %rehashx
        function, which automatically creates aliases for the contents of your
        $PATH.

        If called with no parameters, %alias prints the current alias table
        for your system.  For posix systems, the default aliases are 'cat',
        'cp', 'mv', 'rm', 'rmdir', and 'mkdir', and other platform-specific
        aliases are added.  For windows-based systems, the default aliases are
        'copy', 'ddir', 'echo', 'ls', 'ldir', 'mkdir', 'ren', and 'rmdir'.

        You can see the definition of alias by adding a question mark in the
        end::

          In [1]: cat?
          Repr: <alias cat for 'cat'>"""
        ...
    
    @line_magic
    def unalias(self, parameter_s=...): # -> None:
        """Remove an alias"""
        ...
    
    @line_magic
    def rehashx(self, parameter_s=...): # -> None:
        """Update the alias table with all executable files in $PATH.

        rehashx explicitly checks that every entry in $PATH is a file
        with execute access (os.X_OK).

        Under Windows, it checks executability as a match against a
        '|'-separated string of extensions, stored in the IPython config
        variable win_exec_ext.  This defaults to 'exe|com|bat'.

        This function also resets the root module cache of module completer,
        used on slow filesystems.
        """
        ...
    
    @skip_doctest
    @line_magic
    def pwd(self, parameter_s=...):
        """Return the current working directory path.

        Examples
        --------
        ::

          In [9]: pwd
          Out[9]: '/home/tsuser/sprint/ipython'
        """
        ...
    
    @skip_doctest
    @line_magic
    def cd(self, parameter_s=...):
        """Change the current working directory.

        This command automatically maintains an internal list of directories
        you visit during your IPython session, in the variable ``_dh``. The
        command :magic:`%dhist` shows this history nicely formatted. You can
        also do ``cd -<tab>`` to see directory history conveniently.
        Usage:

          - ``cd 'dir'``: changes to directory 'dir'.
          - ``cd -``: changes to the last visited directory.
          - ``cd -<n>``: changes to the n-th directory in the directory history.
          - ``cd --foo``: change to directory that matches 'foo' in history
          - ``cd -b <bookmark_name>``: jump to a bookmark set by %bookmark
          - Hitting a tab key after ``cd -b`` allows you to tab-complete
            bookmark names.

          .. note::
            ``cd <bookmark_name>`` is enough if there is no directory
            ``<bookmark_name>``, but a bookmark with the name exists.

        Options:

        -q               Be quiet. Do not print the working directory after the
                         cd command is executed. By default IPython's cd
                         command does print this directory, since the default
                         prompts do not display path information.

        .. note::
           Note that ``!cd`` doesn't work for this purpose because the shell
           where ``!command`` runs is immediately discarded after executing
           'command'.

        Examples
        --------
        ::

          In [10]: cd parent/child
          /home/tsuser/parent/child
        """
        ...
    
    @line_magic
    def env(self, parameter_s=...): # -> None:
        """Get, set, or list environment variables.

        Usage:\\

          :``%env``: lists all environment variables/values
          :``%env var``: get value for var
          :``%env var val``: set value for var
          :``%env var=val``: set value for var
          :``%env var=$val``: set value for var, using python expansion if possible
        """
        ...
    
    @line_magic
    def set_env(self, parameter_s): # -> None:
        """Set environment variables.  Assumptions are that either "val" is a
        name in the user namespace, or val is something that evaluates to a
        string.

        Usage:\\
          :``%set_env var val``: set value for var
          :``%set_env var=val``: set value for var
          :``%set_env var=$val``: set value for var, using python expansion if possible
        """
        ...
    
    @line_magic
    def pushd(self, parameter_s=...):
        """Place the current dir on stack and change directory.

        Usage:\\
          %pushd ['dirname']
        """
        ...
    
    @line_magic
    def popd(self, parameter_s=...): # -> None:
        """Change to directory popped off the top of the stack.
        """
        ...
    
    @line_magic
    def dirs(self, parameter_s=...):
        """Return the current directory stack."""
        ...
    
    @line_magic
    def dhist(self, parameter_s=...): # -> None:
        """Print your history of visited directories.

        %dhist       -> print full history\\
        %dhist n     -> print last n entries only\\
        %dhist n1 n2 -> print entries between n1 and n2 (n2 not included)\\

        This history is automatically maintained by the %cd command, and
        always available as the global list variable _dh. You can use %cd -<n>
        to go to directory number <n>.

        Note that most of time, you should view directory history by entering
        cd -<TAB>.

        """
        ...
    
    @skip_doctest
    @line_magic
    def sc(self, parameter_s=...): # -> None:
        """Shell capture - run shell command and capture output (DEPRECATED use !).

        DEPRECATED. Suboptimal, retained for backwards compatibility.

        You should use the form 'var = !command' instead. Example:

         "%sc -l myfiles = ls ~" should now be written as

         "myfiles = !ls ~"

        myfiles.s, myfiles.l and myfiles.n still apply as documented
        below.

        --
        %sc [options] varname=command

        IPython will run the given command using commands.getoutput(), and
        will then update the user's interactive namespace with a variable
        called varname, containing the value of the call.  Your command can
        contain shell wildcards, pipes, etc.

        The '=' sign in the syntax is mandatory, and the variable name you
        supply must follow Python's standard conventions for valid names.

        (A special format without variable name exists for internal use)

        Options:

          -l: list output.  Split the output on newlines into a list before
          assigning it to the given variable.  By default the output is stored
          as a single string.

          -v: verbose.  Print the contents of the variable.

        In most cases you should not need to split as a list, because the
        returned value is a special type of string which can automatically
        provide its contents either as a list (split on newlines) or as a
        space-separated string.  These are convenient, respectively, either
        for sequential processing or to be passed to a shell command.

        For example::

            # Capture into variable a
            In [1]: sc a=ls *py

            # a is a string with embedded newlines
            In [2]: a
            Out[2]: 'setup.py\\nwin32_manual_post_install.py'

            # which can be seen as a list:
            In [3]: a.l
            Out[3]: ['setup.py', 'win32_manual_post_install.py']

            # or as a whitespace-separated string:
            In [4]: a.s
            Out[4]: 'setup.py win32_manual_post_install.py'

            # a.s is useful to pass as a single command line:
            In [5]: !wc -l $a.s
              146 setup.py
              130 win32_manual_post_install.py
              276 total

            # while the list form is useful to loop over:
            In [6]: for f in a.l:
               ...:      !wc -l $f
               ...:
            146 setup.py
            130 win32_manual_post_install.py

        Similarly, the lists returned by the -l option are also special, in
        the sense that you can equally invoke the .s attribute on them to
        automatically get a whitespace-separated string from their contents::

            In [7]: sc -l b=ls *py

            In [8]: b
            Out[8]: ['setup.py', 'win32_manual_post_install.py']

            In [9]: b.s
            Out[9]: 'setup.py win32_manual_post_install.py'

        In summary, both the lists and strings used for output capture have
        the following special attributes::

            .l (or .list) : value as list.
            .n (or .nlstr): value as newline-separated string.
            .s (or .spstr): value as space-separated string.
        """
        ...
    
    @line_cell_magic
    def sx(self, line=..., cell=...): # -> None:
        """Shell execute - run shell command and capture output (!! is short-hand).

        %sx command

        IPython will run the given command using commands.getoutput(), and
        return the result formatted as a list (split on '\\n').  Since the
        output is _returned_, it will be stored in ipython's regular output
        cache Out[N] and in the '_N' automatic variables.

        Notes:

        1) If an input line begins with '!!', then %sx is automatically
        invoked.  That is, while::

          !ls

        causes ipython to simply issue system('ls'), typing::

          !!ls

        is a shorthand equivalent to::

          %sx ls

        2) %sx differs from %sc in that %sx automatically splits into a list,
        like '%sc -l'.  The reason for this is to make it as easy as possible
        to process line-oriented shell output via further python commands.
        %sc is meant to provide much finer control, but requires more
        typing.

        3) Just like %sc -l, this is a list with special attributes:
        ::

          .l (or .list) : value as list.
          .n (or .nlstr): value as newline-separated string.
          .s (or .spstr): value as whitespace-separated string.

        This is very useful when trying to use such lists as arguments to
        system commands."""
        ...
    
    system = ...
    bang = ...
    @line_magic
    def bookmark(self, parameter_s=...): # -> None:
        """Manage IPython's bookmark system.

        %bookmark <name>       - set bookmark to current dir
        %bookmark <name> <dir> - set bookmark to <dir>
        %bookmark -l           - list all bookmarks
        %bookmark -d <name>    - remove bookmark
        %bookmark -r           - remove all bookmarks

        You can later on access a bookmarked folder with::

          %cd -b <name>

        or simply '%cd <name>' if there is no directory called <name> AND
        there is such a bookmark defined.

        Your bookmarks persist through IPython sessions, but they are
        associated with each profile."""
        ...
    
    @line_magic
    def pycat(self, parameter_s=...): # -> None:
        """Show a syntax-highlighted file through a pager.

        This magic is similar to the cat utility, but it will assume the file
        to be Python source and will show it with syntax highlighting.

        This magic command can either take a local filename, an url,
        an history range (see %history) or a macro as argument.

        If no parameter is given, prints out history of current session up to
        this point. ::

        %pycat myscript.py
        %pycat 7-27
        %pycat myMacro
        %pycat http://www.example.com/myscript.py
        """
        ...
    
    @magic_arguments.magic_arguments()
    @magic_arguments.argument('-a', '--append', action='store_true', default=False, help='Append contents of the cell to an existing file. ' 'The file will be created if it does not exist.')
    @magic_arguments.argument('filename', type=str, help='file to write')
    @cell_magic
    def writefile(self, line, cell): # -> None:
        """Write the contents of the cell to a file.

        The file will be overwritten unless the -a (--append) flag is specified.
        """
        ...
    





from io import BufferedWriter, BytesIO
from .colorbar import Colorbar
import numpy as np
from typing import Callable, Literal, overload
from ._typing import *
from .text import Text
from .gridspec import GridSpec, SubplotSpec
from .backend_bases import FigureCanvasBase, MouseButton, MouseEvent, RendererBase
from .colors import Colormap, Normalize
from .legend import Legend
from .layout_engine import LayoutEngine
from .image import FigureImage
from .transforms import BboxBase
from .axes import Axes
from .artist import Artist, _finalize_rasterization, allow_rasterization
from mpl_toolkits.mplot3d import Axes3D
from .projections.geo import AitoffAxes, HammerAxes, LambertAxes, MollweideAxes
from .projections.polar import PolarAxes

class _AxesStack:
    def __init__(self) -> None: ...
    def as_list(self) -> list[Axes]: ...
    def remove(self, a: Axes) -> None: ...
    def bubble(self, a: Axes) -> None: ...
    def add(self, a: Axes) -> None: ...
    def current(self) -> None: ...

class SubplotParams:
    def __init__(
        self,
        left: float = ...,
        bottom: float = ...,
        right: float = ...,
        top: float = ...,
        wspace: float = ...,
        hspace: float = ...,
    ) -> None: ...
    validate = ...
    def update(
        self,
        left: float | None = ...,
        bottom: float | None = ...,
        right: float | None = ...,
        top: float | None = ...,
        wspace: float | None = ...,
        hspace: float | None = ...,
    ) -> None: ...

class FigureBase(Artist):
    def __init__(self, **kwargs) -> None: ...
    def autofmt_xdate(
        self,
        bottom: float = 0.2,
        rotation: int = 30,
        ha: Literal["left", "center", "right"] = "right",
        which: Literal["major", "minor", "both"] = "major",
    ) -> None: ...
    def get_children(self) -> list[Artist]: ...
    def contains(self, mouseevent: MouseEvent) -> bool: ...
    def get_window_extent(self, renderer: RendererBase = ..., *args, **kwargs): ...
    def suptitle(self, t: str, **kwargs) -> Text: ...
    def supxlabel(self, t: str, **kwargs) -> Text: ...
    def supylabel(self, t: str, **kwargs) -> Text: ...
    def get_edgecolor(self): ...
    def get_facecolor(self): ...
    def get_frameon(self) -> bool: ...
    def set_linewidth(self, linewidth: float) -> None: ...
    def get_linewidth(self) -> float: ...
    def set_edgecolor(self, color: Color): ...
    def set_facecolor(self, color: str) -> None: ...
    def set_frameon(self, b: bool): ...
    frameon = ...
    def add_artist(self, artist: Artist, clip: bool = False) -> Artist: ...
    def add_axes(self, *args, **kwargs) -> Axes: ...
    @overload
    def add_subplot(self, *args, projection: Literal['3d'], **kwargs) -> Axes3D: ...
    @overload
    def add_subplot(self, *args, projection: Literal['aitoff'], **kwargs) -> AitoffAxes: ...
    @overload
    def add_subplot(self, *args, projection: Literal['hammer'], **kwargs) -> HammerAxes: ...
    @overload
    def add_subplot(self, *args, projection: Literal['lambert'], **kwargs) -> LambertAxes: ...
    @overload
    def add_subplot(self, *args, projection: Literal['mollweide'], **kwargs) -> MollweideAxes: ...
    @overload
    def add_subplot(self, *args, projection: Literal['polar'], **kwargs) -> PolarAxes: ...
    @overload
    def add_subplot(self, *args, **kwargs) -> Axes: ...
    @overload
    def subplots(
        self,
        nrows: Literal[1] = ...,
        ncols: Literal[1] = ...,
        *,
        squeeze: Literal[False],
        sharex: bool | Literal["none", "all", "row", "col"] = ...,
        sharey: bool | Literal["none", "all", "row", "col"] = ...,
        subplot_kw: dict = ...,
        gridspec_kw: dict = ...
    ) -> list[Axes]: ...
    @overload
    def subplots(
        self,
        nrows: int = ...,
        ncols: int = ...,
        *,
        squeeze: Literal[False],
        sharex: bool | Literal["none", "all", "row", "col"] = ...,
        sharey: bool | Literal["none", "all", "row", "col"] = ...,
        subplot_kw: dict = ...,
        gridspec_kw: dict = ...
    ) -> list[list[Axes]]: ...
    @overload
    def subplots(
        self,
        nrows: Literal[1] = ...,
        ncols: Literal[1] = ...,
        *,
        squeeze: Literal[True] = ...,
        sharex: bool | Literal["none", "all", "row", "col"] = ...,
        sharey: bool | Literal["none", "all", "row", "col"] = ...,
        subplot_kw: dict = ...,
        gridspec_kw: dict = ...
    ) -> Axes: ...
    @overload
    def subplots(
        self,
        nrows: int,
        ncols: int,
        *,
        squeeze: bool = ...,
        sharex: bool | Literal["none", "all", "row", "col"] = ...,
        sharey: bool | Literal["none", "all", "row", "col"] = ...,
        subplot_kw: dict = ...,
        gridspec_kw: dict = ...
    ) -> list[Axes]: ...
    def delaxes(self, ax: Axes) -> None: ...
    def clear(self, keep_observers: bool = False) -> None: ...
    def clf(self, keep_observers: bool = False) -> None: ...
    def legend(self, *args, **kwargs) -> Legend: ...
    def text(
        self, x: float, y: float, s: str, fontdict: dict = ..., **kwargs
    ) -> Text: ...
    def colorbar(
        self, mappable, cax: Axes = ..., ax=..., use_gridspec: bool = ..., **kwargs
    ) -> Colorbar: ...
    def subplots_adjust(
        self,
        left: float = ...,
        bottom: float = ...,
        right: float = ...,
        top: float = ...,
        wspace: float = ...,
        hspace: float = ...,
    ) -> None: ...
    def align_xlabels(self, axs: list[Axes] = ...) -> None: ...
    def align_ylabels(self, axs: list[Axes] = ...) -> None: ...
    def align_labels(self, axs: list[Axes] = ...) -> None: ...
    def add_gridspec(self, nrows: int = 1, ncols: int = 1, **kwargs) -> GridSpec: ...
    def subfigures(
        self,
        nrows: int = 1,
        ncols: int = 1,
        squeeze: bool = True,
        wspace: float | None = None,
        hspace: float | None = None,
        width_ratios: ArrayLike = ...,
        height_ratios: ArrayLike = ...,
        **kwargs
    ): ...
    def add_subfigure(self, subplotspec: SubplotSpec, **kwargs) -> SubFigure: ...
    def sca(self, a: Axes) -> Axes: ...
    def gca(self) -> Axes: ...
    def get_default_bbox_extra_artists(self): ...
    def get_tightbbox(
        self,
        renderer: RendererBase = ...,
        bbox_extra_artists: list[Artist] | None = ...,
    ) -> BboxBase: ...
    def subplot_mosaic(
        self,
        mosaic: list | str,
        *,
        sharex: bool = ...,
        sharey: bool = ...,
        subplot_kw: dict = ...,
        gridspec_kw: dict = ...,
        empty_sentinel: object = ...
    ) -> dict[Text, Axes]: ...

class Figure(FigureBase):

    callbacks = ...
    def __str__(self) -> str: ...
    def __repr__(self): ...
    def __init__(
        self,
        figsize: tuple[float, float] = ...,
        dpi: float = ...,
        facecolor: Color = ...,
        edgecolor: Color = ...,
        linewidth: float = ...,
        frameon: bool = ...,
        subplotpars: SubplotParams = ...,
        tight_layout: bool | dict = ...,
        constrained_layout: bool = ...,
        *,
        layout: LayoutEngine
        | Literal["constrained", "compressed", "tight"]
        | None = ...,
        **kwargs
    ) -> None: ...
    def set_layout_engine(
        self,
        layout: LayoutEngine | Literal["constrained", "compressed", "tight"] = ...,
        **kwargs: dict
    ) -> None: ...
    def get_layout_engine(self) -> None: ...
    def show(self, warn: bool = True) -> None: ...
    @property
    def axes(self) -> list[Axes]: ...
    get_axes = ...
    dpi = ...
    def get_tight_layout(self) -> bool: ...
    def set_tight_layout(
        self, tight: bool | Literal["w_pad", "h_pad", "rect"] | None
    ): ...
    def get_constrained_layout(self) -> bool: ...
    def set_constrained_layout(self, constrained: bool | dict | None): ...
    def set_constrained_layout_pads(self, **kwargs) -> None: ...
    def get_constrained_layout_pads(self, relative: bool = ...): ...
    def set_canvas(self, canvas: FigureCanvasBase) -> None: ...
    canvas: FigureCanvasBase

    def figimage(
        self,
        X: ArrayLike,
        xo: int = ...,
        yo: int = ...,
        alpha: None | float = ...,
        norm: Normalize = ...,
        cmap: str | Colormap = ...,
        vmin: float = ...,
        vmax: float = ...,
        origin: Literal["upper", "lower"] = ...,
        resize: bool = ...,
        **kwargs
    ) -> FigureImage: ...
    def set_size_inches(
        self, w: float, h: float = ..., forward: bool = True
    ) -> None: ...
    def get_size_inches(self) -> np.ndarray: ...
    def get_figwidth(self): ...
    def get_figheight(self): ...
    def get_dpi(self) -> float: ...
    def set_dpi(self, val: float) -> None: ...
    def set_figwidth(self, val: float, forward: bool = ...)-> None: ...
    def set_figheight(self, val: float, forward: bool = ...)-> None: ...
    def clear(self, keep_observers: bool = ...) -> None: ...
    @allow_rasterization
    def draw(self, renderer) -> None: ...
    def draw_without_rendering(self) -> None: ...
    def draw_artist(self, a: Artist) -> None: ...
    def __getstate__(self): ...
    def __setstate__(self, state) -> None: ...
    def add_axobserver(self, func: Callable) -> None: ...
    def savefig(
        self,
        fname: str | PathLike | FileLike | BytesIO | BufferedWriter,
        *,
        transparent: bool = ...,
        **kwargs
    ) -> None: ...
    def ginput(
        self,
        n: int = ...,
        timeout: float = ...,
        show_clicks: bool = ...,
        mouse_add: MouseButton | None = ...,
        mouse_pop: MouseButton | None = ...,
        mouse_stop: MouseButton | None = ...,
    ) -> list[tuple[float, float]]: ...
    def waitforbuttonpress(self, timeout=...) -> None: ...
    def execute_constrained_layout(self, renderer=...) -> None: ...
    def tight_layout(
        self,
        *,
        pad: float = 1.08,
        h_pad: float = ...,
        w_pad: float = ...,
        rect: tuple[float, float, float, float] = ...
    ) -> None: ...

def figaspect(arg: float): ...

class SubFigure(FigureBase):

    callbacks = ...
    def __init__(
        self,
        parent: FigureBase,
        subplotspec: SubplotSpec,
        *,
        facecolor: Color = ...,
        edgecolor: Color = ...,
        linewidth: float = ...,
        frameon: bool = ...,
        **kwargs
    ) -> None: ...
    @property
    def dpi(self) -> float: ...
    @dpi.setter
    def dpi(self, value: float): ...
    def get_dpi(self) -> float: ...
    def set_dpi(self, val: float)-> None: ...
    def get_constrained_layout(self) -> bool: ...
    def get_constrained_layout_pads(self, relative: bool = ...): ...
    def get_layout_engine(self) -> LayoutEngine: ...
    @property
    def axes(self) -> list[Axes]: ...
    get_axes = ...
    def draw(self, renderer: RendererBase) -> None: ...



"""
This type stub file was generated by pyright.
"""

import unittest

"""Decorators for labeling test objects.

Decorators that merely return a modified version of the original function
object are straightforward.  Decorators that return a new function object need
to use nose.tools.make_decorator(original_function)(decorator) in returning the
decorator, in order to preserve metadata such as function name, setup and
teardown functions and so on - see nose.tools for more information.

This module provides a set of useful decorators meant to be ready to use in
your own tests.  See the bottom of the file for the ready-made ones, and if you
find yourself writing a new one that may be of generic use, add it here.

Included decorators:


Lightweight testing that remains unittest-compatible.

- An @as_unittest decorator can be used to tag any normal parameter-less
  function as a unittest TestCase.  Then, both nose and normal unittest will
  recognize it as such.  This will make it easier to migrate away from Nose if
  we ever need/want to while maintaining very lightweight tests.

NOTE: This file contains IPython-specific decorators. Using the machinery in
IPython.external.decorators, we import either numpy.testing.decorators if numpy is
available, OR use equivalent code in IPython.external._decorators, which
we've copied verbatim from numpy.

"""
def as_unittest(func): # -> Type[Tester]:
    """Decorator to make a simple function into a normal test via unittest."""
    class Tester(unittest.TestCase):
        ...
    
    

def skipif(skip_condition, msg=...): # -> MarkDecorator:
    """Make function raise SkipTest exception if skip_condition is true

    Parameters
    ----------

    skip_condition : bool or callable
      Flag to determine whether to skip test. If the condition is a
      callable, it is used at runtime to dynamically make the decision. This
      is useful for tests that may require costly imports, to delay the cost
      until the test suite is actually executed.
    msg : string
      Message to give on raising a SkipTest exception.

    Returns
    -------
    decorator : function
      Decorator, which, when applied to a function, causes SkipTest
      to be raised when the skip_condition was True, and the function
      to be called normally otherwise.
    """
    ...

def skip(msg=...): # -> MarkDecorator:
    """Decorator factory - mark a test function for skipping from test suite.

    Parameters
    ----------
      msg : string
        Optional message to be added.

    Returns
    -------
       decorator : function
         Decorator, which, when applied to a function, causes SkipTest
         to be raised, with the optional message added.
      """
    ...

def onlyif(condition, msg): # -> MarkDecorator:
    """The reverse from skipif, see skipif for details."""
    ...

def module_not_available(module):
    """Can module be imported?  Returns true if module does NOT import.

    This is used to make a decorator to skip tests that require module to be
    available, but delay the 'import numpy' to test execution time.
    """
    ...

skip_win32 = ...
skip_linux = ...
skip_osx = ...
skip_if_not_win32 = ...
skip_if_not_linux = ...
_x11_skip_cond = ...
_x11_skip_msg = ...
skip_if_no_x11 = ...
skip_without = ...
skipif_not_numpy = ...
skipif_not_matplotlib = ...
null_deco = ...
f = ...
onlyif_unicode_paths = ...
def onlyif_cmds_exist(*commands): # -> MarkDecorator | ((f: Unknown) -> Unknown):
    """
    Decorator to skip test when at least one of `commands` is not found.
    """
    ...





import numpy as np


def min_pos(X: np.ndarray) -> float:
    """Find the minimum value of an array over positive values
    Returns the maximum representable value of the input dtype if none of the
    values are positive.
    """
    ...



def cholesky_delete(L: np.ndarray, go_out: int) -> None: ...



from typing import NamedTuple

from matplotlib.axes import Axes as PlotAxes
from matplotlib.lines import Line2D

from pandas.plotting._matplotlib.core import LinePlot as LinePlot

class BoxPlot(LinePlot):
    class BP(NamedTuple):
        ax: PlotAxes
        lines: dict[str, list[Line2D]]
    return_type = ...
    def __init__(self, data, return_type: str = ..., **kwargs) -> None: ...
    def maybe_color_bp(self, bp) -> None: ...
    @property
    def orientation(self): ...
    @property
    def result(self): ...

def boxplot(
    data,
    column=...,
    by=...,
    ax=...,
    fontsize=...,
    rot: int = ...,
    grid: bool = ...,
    figsize=...,
    layout=...,
    return_type=...,
    **kwds,
): ...
def boxplot_frame(
    self,
    column=...,
    by=...,
    ax=...,
    fontsize=...,
    rot: int = ...,
    grid: bool = ...,
    figsize=...,
    layout=...,
    return_type=...,
    **kwds,
): ...
def boxplot_frame_groupby(
    grouped,
    subplots: bool = ...,
    column=...,
    fontsize=...,
    rot: int = ...,
    grid: bool = ...,
    ax=...,
    figsize=...,
    layout=...,
    sharex: bool = ...,
    sharey: bool = ...,
    **kwds,
): ...



from numpy.typing import ArrayLike, NDArray

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from ..visual import CompoundVisual
from ..line import ArrowVisual
from ..markers import MarkersVisual
from ...color import Colormap
from . import layouts

class GraphVisual(CompoundVisual):

    _arrow_attributes = ...
    _arrow_kwargs = ...
    _node_kwargs = ...

    _arrow_kw_trans = ...
    _node_kw_trans = ...
    _node_properties_args = ...

    def __init__(
        self,
        adjacency_mat: ArrayLike | NDArray | None = None,
        directed: bool = False,
        layout: str | None = None,
        animate: bool = False,
        line_color: str | Colormap | None = None,
        line_width: float | None = None,
        arrow_type: str | None = None,
        arrow_size: float | None = None,
        node_symbol: str | None = None,
        node_size: float | None = None,
        border_color: str | Colormap | None = None,
        face_color: str | Colormap | None = None,
        border_width: float | None = None,
    ): ...
    @property
    def adjacency_matrix(self): ...
    @property
    def layout(self): ...
    @layout.setter
    def layout(self, value): ...
    @property
    def directed(self): ...
    @directed.setter
    def directed(self, value): ...
    @property
    def animate(self): ...
    @animate.setter
    def animate(self, value): ...
    def animate_layout(self): ...
    def set_final_layout(self): ...
    def reset_layout(self): ...
    def set_data(self, adjacency_mat: NDArray | None = None, **kwargs): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.io.matlab.mio5_utils, version: unspecified
import typing
import builtins as _mod_builtins
import scipy.sparse.csc as _mod_scipy_sparse_csc

class VarHeader5(_mod_builtins.object):
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    @property
    def dims(self) -> typing.Any:
        ...
    
    @property
    def is_global(self) -> typing.Any:
        ...
    
    @property
    def is_logical(self) -> typing.Any:
        ...
    
    @property
    def mclass(self) -> typing.Any:
        ...
    
    @property
    def name(self) -> typing.Any:
        ...
    
    @property
    def nzmax(self) -> typing.Any:
        ...
    
    def set_dims(self) -> typing.Any:
        ' Allow setting of dimensions from python\n\n        This is for constructing headers for tests\n        '
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

class VarReader5(_mod_builtins.object):
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __pyx_vtable__: PyCapsule
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def array_from_header(self) -> typing.Any:
        ' Read array of any class, given matrix `header`\n\n        Parameters\n        ----------\n        header : VarHeader5\n           array header object\n        process : int, optional\n           If not zero, apply post-processing on returned array\n           \n        Returns\n        -------\n        arr : array or sparse array\n           read array\n        '
        ...
    
    @property
    def is_swapped(self) -> typing.Any:
        ...
    
    @property
    def little_endian(self) -> typing.Any:
        ...
    
    def read_cells(self) -> typing.Any:
        ' Read cell array from stream '
        ...
    
    def read_char(self) -> typing.Any:
        ' Read char matrices from stream as arrays\n\n        Matrices of char are likely to be converted to matrices of\n        string by later processing in ``array_from_header``\n        '
        ...
    
    def read_fieldnames(self) -> typing.Any:
        'Read fieldnames for struct-like matrix.'
        ...
    
    def read_full_tag(self) -> typing.Any:
        ' Python method for reading full u4, u4 tag from stream\n\n        Returns\n        -------\n        mdtype : int32\n           matlab data type code\n        byte_count : int32\n           number of data bytes following\n\n        Notes\n        -----\n        Assumes tag is in fact full, that is, is not a small data\n        element.  This means it can skip some checks and makes it\n        slightly faster than ``read_tag``\n        '
        ...
    
    def read_header(self) -> typing.Any:
        ' Return matrix header for current stream position\n\n        Returns matrix headers at top level and sub levels\n\n        Parameters\n        ----------\n        check_stream_limit : if True, then if the returned header\n        is passed to array_from_header, it will be verified that\n        the length of the uncompressed data is not overlong (which\n        can indicate .mat file corruption)\n        '
        ...
    
    def read_numeric(self) -> typing.Any:
        ' Read numeric data element into ndarray\n\n        Reads element, then casts to ndarray.\n\n        The type of the array is usually given by the ``mdtype`` returned via\n        ``read_element``.  Sparse logical arrays are an exception, where the\n        type of the array may be ``np.bool_`` even if the ``mdtype`` claims the\n        data is of float64 type.\n\n        Parameters\n        ----------\n        copy : bool, optional\n            Whether to copy the array before returning.  If False, return array\n            backed by bytes read from file.\n        nnz : int, optional\n            Number of non-zero values when reading numeric data from sparse\n            matrices.  -1 if not reading sparse matrices, or to disable check\n            for bytes data instead of declared data type (see Notes).\n\n        Returns\n        -------\n        arr : array\n            Numeric array\n\n        Notes\n        -----\n        MATLAB apparently likes to store sparse logical matrix data as bytes\n        instead of miDOUBLE (float64) data type, even though the data element\n        still declares its type as miDOUBLE.  We can guess this has happened by\n        looking for the length of the data compared to the expected number of\n        elements, using the `nnz` input parameter.\n        '
        ...
    
    def read_opaque(self) -> typing.Any:
        " Read opaque (function workspace) type\n\n        Looking at some mat files, the structure of this type seems to\n        be:\n\n        * array flags as usual (already read into `hdr`)\n        * 3 int8 strings\n        * a matrix\n\n        Then there's a matrix at the end of the mat file that seems have\n        the anonymous founction workspaces - we load it as\n        ``__function_workspace__``\n\n        See the comments at the beginning of ``mio5.py``\n        "
        ...
    
    def read_real_complex(self) -> typing.Any:
        ' Read real / complex matrices from stream '
        ...
    
    def read_struct(self) -> typing.Any:
        ' Read struct or object array from stream\n\n        Objects are just structs with an extra field *classname*,\n        defined before (this here) struct format structure\n        '
        ...
    
    def read_tag(self) -> typing.Any:
        ' Read tag mdtype and byte_count\n\n        Does necessary swapping and takes account of SDE formats.\n\n        See also ``read_full_tag`` method.\n        \n        Returns\n        -------\n        mdtype : int\n           matlab data type code\n        byte_count : int\n           number of bytes following that comprise the data\n        tag_data : None or str\n           Any data from the tag itself.  This is None for a full tag,\n           and string length `byte_count` if this is a small data\n           element.\n        '
        ...
    
    def set_stream(self) -> typing.Any:
        ' Set stream of best type from file-like `fobj`\n\n        Called from Python when initiating a variable read\n        '
        ...
    
    def shape_from_header(self) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

__doc__: str
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_VarHeader5() -> typing.Any:
    ...

__test__: dict
def asbytes(s) -> typing.Any:
    ...

def asstr(s) -> typing.Any:
    ...

def byteswap_u4() -> typing.Any:
    ...

def chars_to_strings() -> typing.Any:
    " Convert final axis of char array to strings\n\n    Parameters\n    ----------\n    in_arr : array\n       dtype of 'U1'\n\n    Returns\n    -------\n    str_arr : array\n       dtype of 'UN' where N is the length of the last dimension of\n       ``arr``\n    "
    ...

csc_matrix = _mod_scipy_sparse_csc.csc_matrix
def pycopy(x) -> typing.Any:
    "Shallow copy operation on arbitrary Python objects.\n\n    See the module's __doc__ string for more info.\n    "
    ...

def squeeze_element() -> typing.Any:
    ' Return squeezed element\n\n    The returned object may not be an ndarray - for example if we do\n    ``arr.item`` to return a ``mat_struct`` object from a struct array '
    ...

swapped_code: str
def __getattr__(name) -> typing.Any:
    ...




# -*- coding: utf-8 -*-

from ctypes import (
    cdll,
    util,
    Structure,
    cast,
    byref,
    POINTER,
    CFUNCTYPE,
    c_int,
    c_long,
    c_ulong,
    c_ushort,
    c_wchar,
    c_uint32,
    c_double,
    c_uint,
    c_float,
    c_void_p,
    c_char_p,
    c_bool,
    c_buffer,
    c_ubyte,
    c_byte,
    c_int8,
    c_int16,
    c_int32,
    c_int64,
    c_short,
    c_longlong,
    c_size_t,
    sizeof,
    c_uint8,
    c_longdouble,
    c_char,
    c_ulonglong,
    py_object,
    alignment,
    ArgumentError,
)

import platform
import struct
import sys

# Based on Pyglet code

##############################################################################
# cocoatypes.py

__LP64__ = ...
__i386__ = ...

PyObjectEncoding: str = ...

def encoding_for_ctype(vartype): ...

NSIntegerEncoding = ...
NSUIntegerEncoding = ...
CGFloatEncoding = ...

CGImageEncoding: str = ...
NSZoneEncoding: str = ...

class NSPoint(Structure):
    _fields_: list = ...

CGPoint = NSPoint

class NSSize(Structure):
    _fields_: list = ...

CGSize = NSSize

class NSRect(Structure):
    _fields_: list = ...

CGRect = NSRect

NSTimeInterval = ...
CFIndex = ...
UniChar = ...
unichar = ...
CGGlyph = ...

class CFRange(Structure):
    _fields_: list = ...

class NSRange(Structure):
    _fields_: list = ...

CFTypeID = ...
CFNumberType = ...

##############################################################################
# runtime.py

__LP64__ = ...
__i386__ = ...

objc = ...

objc.class_addIvar.restype = ...
objc.class_addIvar.argtypes = ...

objc.class_addMethod.restype = ...

objc.class_addProtocol.restype = ...
objc.class_addProtocol.argtypes = ...

objc.class_conformsToProtocol.restype = ...
objc.class_conformsToProtocol.argtypes = ...

objc.class_copyIvarList.restype = ...
objc.class_copyIvarList.argtypes = ...

objc.class_copyMethodList.restype = ...
objc.class_copyMethodList.argtypes = ...

objc.class_copyPropertyList.restype = ...
objc.class_copyPropertyList.argtypes = ...

objc.class_copyProtocolList.restype = ...
objc.class_copyProtocolList.argtypes = ...

objc.class_createInstance.restype = ...
objc.class_createInstance.argtypes = ...

objc.class_getClassMethod.restype = ...
objc.class_getClassMethod.argtypes = ...

objc.class_getClassVariable.restype = ...
objc.class_getClassVariable.argtypes = ...

objc.class_getInstanceMethod.restype = ...
objc.class_getInstanceMethod.argtypes = ...

objc.class_getInstanceSize.restype = ...
objc.class_getInstanceSize.argtypes = ...

objc.class_getInstanceVariable.restype = ...
objc.class_getInstanceVariable.argtypes = ...

objc.class_getIvarLayout.restype = ...
objc.class_getIvarLayout.argtypes = ...

objc.class_getMethodImplementation.restype = ...
objc.class_getMethodImplementation.argtypes = ...

objc.class_getName.restype = ...
objc.class_getName.argtypes = ...

objc.class_getProperty.restype = ...
objc.class_getProperty.argtypes = ...

objc.class_getSuperclass.restype = ...
objc.class_getSuperclass.argtypes = ...

objc.class_getVersion.restype = ...
objc.class_getVersion.argtypes = ...

objc.class_getWeakIvarLayout.restype = ...
objc.class_getWeakIvarLayout.argtypes = ...

objc.class_isMetaClass.restype = ...
objc.class_isMetaClass.argtypes = ...

objc.class_replaceMethod.restype = ...
objc.class_replaceMethod.argtypes = ...

objc.class_respondsToSelector.restype = ...
objc.class_respondsToSelector.argtypes = ...

objc.class_setIvarLayout.restype = ...
objc.class_setIvarLayout.argtypes = ...

objc.class_setSuperclass.restype = ...
objc.class_setSuperclass.argtypes = ...

objc.class_setVersion.restype = ...
objc.class_setVersion.argtypes = ...

objc.class_setWeakIvarLayout.restype = ...
objc.class_setWeakIvarLayout.argtypes = ...

objc.ivar_getName.restype = ...
objc.ivar_getName.argtypes = ...

objc.ivar_getOffset.restype = ...
objc.ivar_getOffset.argtypes = ...

objc.ivar_getTypeEncoding.restype = ...
objc.ivar_getTypeEncoding.argtypes = ...

objc.method_copyArgumentType.restype = ...
objc.method_copyArgumentType.argtypes = ...

objc.method_copyReturnType.restype = ...
objc.method_copyReturnType.argtypes = ...

objc.method_exchangeImplementations.restype = ...
objc.method_exchangeImplementations.argtypes = ...

objc.method_getArgumentType.restype = ...
objc.method_getArgumentType.argtypes = ...

objc.method_getImplementation.restype = ...
objc.method_getImplementation.argtypes = ...

objc.method_getName.restype = ...
objc.method_getName.argtypes = ...

objc.method_getNumberOfArguments.restype = ...
objc.method_getNumberOfArguments.argtypes = ...

objc.method_getReturnType.restype = ...
objc.method_getReturnType.argtypes = ...

objc.method_getTypeEncoding.restype = ...
objc.method_getTypeEncoding.argtypes = ...

objc.method_setImplementation.restype = ...
objc.method_setImplementation.argtypes = ...

objc.objc_allocateClassPair.restype = ...
objc.objc_allocateClassPair.argtypes = ...

objc.objc_copyProtocolList.restype = ...
objc.objc_copyProtocolList.argtypes = ...

objc.objc_getAssociatedObject.restype = ...
objc.objc_getAssociatedObject.argtypes = ...

objc.objc_getClass.restype = ...
objc.objc_getClass.argtypes = ...

objc.objc_getClassList.restype = ...
objc.objc_getClassList.argtypes = ...

objc.objc_getMetaClass.restype = ...
objc.objc_getMetaClass.argtypes = ...

objc.objc_getProtocol.restype = ...
objc.objc_getProtocol.argtypes = ...

objc.objc_registerClassPair.restype = ...
objc.objc_registerClassPair.argtypes = ...

objc.objc_removeAssociatedObjects.restype = ...
objc.objc_removeAssociatedObjects.argtypes = ...

objc.objc_setAssociatedObject.restype = ...
objc.objc_setAssociatedObject.argtypes = ...

objc.object_copy.restype = ...
objc.object_copy.argtypes = ...

objc.object_dispose.restype = ...
objc.object_dispose.argtypes = ...

objc.object_getClass.restype = ...
objc.object_getClass.argtypes = ...

objc.object_getClassName.restype = ...
objc.object_getClassName.argtypes = ...

objc.object_getInstanceVariable.restype = ...
objc.object_getInstanceVariable.argtypes = ...

objc.object_getIvar.restype = ...
objc.object_getIvar.argtypes = ...

objc.object_setClass.restype = ...
objc.object_setClass.argtypes = ...

objc.object_setInstanceVariable.restype = ...

objc.object_setIvar.restype = ...
objc.object_setIvar.argtypes = ...

objc.property_getAttributes.restype = ...
objc.property_getAttributes.argtypes = ...

objc.property_getName.restype = ...
objc.property_getName.argtypes = ...

objc.protocol_conformsToProtocol.restype = ...
objc.protocol_conformsToProtocol.argtypes = ...

class OBJC_METHOD_DESCRIPTION(Structure):
    _fields_ = ...

objc.protocol_copyMethodDescriptionList.restype = ...
objc.protocol_copyMethodDescriptionList.argtypes = ...

objc.protocol_copyPropertyList.restype = ...
objc.protocol_copyPropertyList.argtypes = ...

objc.protocol_copyProtocolList = ...
objc.protocol_copyProtocolList.argtypes = ...

objc.protocol_getMethodDescription.restype = OBJC_METHOD_DESCRIPTION
objc.protocol_getMethodDescription.argtypes = ...

objc.protocol_getName.restype = ...
objc.protocol_getName.argtypes = ...

objc.sel_getName.restype = ...
objc.sel_getName.argtypes = ...

objc.sel_isEqual.restype = ...
objc.sel_isEqual.argtypes = ...

objc.sel_registerName.restype = ...
objc.sel_registerName.argtypes = ...

def ensure_bytes(x): ...
def get_selector(name): ...
def get_class(name): ...
def get_object_class(obj): ...
def get_metaclass(name): ...
def get_superclass_of_object(obj): ...
def x86_should_use_stret(restype): ...
def should_use_fpret(restype): ...
def send_message(receiver, selName, *args, **kwargs): ...

class OBJC_SUPER(Structure):
    _fields_: list = ...

OBJC_SUPER_PTR = ...

def send_super(receiver, selName, *args, **kwargs): ...

cfunctype_table: dict = ...

def parse_type_encoding(encoding): ...
def cfunctype_for_encoding(encoding): ...
def create_subclass(superclass, name): ...
def register_subclass(subclass): ...
def add_method(cls, selName, method, types): ...
def add_ivar(cls, name, vartype): ...
def set_instance_variable(obj, varname, value, vartype): ...
def get_instance_variable(obj, varname, vartype): ...

class ObjCMethod(object):

    typecodes: dict = ...
    cfunctype_table: dict = ...

    def __init__(self, method): ...
    def ctype_for_encoding(self, encoding): ...
    def get_prototype(self): ...
    def __repr__(self): ...
    def get_callable(self): ...
    def __call__(self, objc_id, *args): ...

class ObjCBoundMethod(object):
    def __init__(self, method, objc_id): ...
    def __repr__(self): ...
    def __call__(self, *args): ...

class ObjCClass(object):
    _registered_classes: dict = ...

    def __new__(cls, class_name_or_ptr): ...
    def __repr__(self): ...
    def cache_instance_methods(self): ...
    def cache_class_methods(self): ...
    def get_instance_method(self, name): ...
    def get_class_method(self, name): ...
    def __getattr__(self, name): ...

class ObjCInstance(object):
    _cached_objects: dict = ...

    def __new__(cls, object_ptr): ...
    def __repr__(self): ...
    def __getattr__(self, name): ...

def convert_method_arguments(encoding, args): ...

class ObjCSubclass(object):
    def __init__(self, superclass, name, register=True): ...
    def register(self): ...
    def add_ivar(self, varname, vartype): ...
    def add_method(self, method, name, encoding): ...
    def add_class_method(self, method, name, encoding): ...
    def rawmethod(self, encoding): ...
    def method(self, encoding): ...
    def classmethod(self, encoding): ...

# XXX This causes segfaults in all backends (yikes!), and makes it so that
# pyglet can't even be loaded. We'll just have to live with leaks for now,
# which is probably alright since we only use the
# NSFontManager.sharedFontManager class currently.

# class DeallocationObserver_Implementation(object):
#    DeallocationObserver = ObjCSubclass('NSObject', 'DeallocationObserver',
#                                        register=False)
#    DeallocationObserver.add_ivar('observed_object', c_void_p)
#    DeallocationObserver.register()
#
#    @DeallocationObserver.rawmethod('@@')
#    def initWithObject_(self, cmd, anObject):
#        self = send_super(self, 'init')
#        self = self.value
#        set_instance_variable(self, 'observed_object', anObject, c_void_p)
#        return self
#
#    @DeallocationObserver.rawmethod('v')
#    def dealloc(self, cmd):
#        anObject = get_instance_variable(self, 'observed_object', c_void_p)
#        ObjCInstance._cached_objects.pop(anObject, None)
#        send_super(self, 'dealloc')
#
#    @DeallocationObserver.rawmethod('v')
#    def finalize(self, cmd):
#        anObject = get_instance_variable(self, 'observed_object', c_void_p)
#        ObjCInstance._cached_objects.pop(anObject, None)
#        send_super(self, 'finalize')

##############################################################################
# cocoalibs.py

cf = ...

kCFStringEncodingUTF8: int = ...

CFAllocatorRef = ...
CFStringEncoding = ...

cf.CFStringCreateWithCString.restype = ...
cf.CFStringCreateWithCString.argtypes = ...

cf.CFRelease.restype = ...
cf.CFRelease.argtypes = ...

cf.CFStringGetLength.restype = ...
cf.CFStringGetLength.argtypes = ...

cf.CFStringGetMaximumSizeForEncoding.restype = ...
cf.CFStringGetMaximumSizeForEncoding.argtypes = ...

cf.CFStringGetCString.restype = ...
cf.CFStringGetCString.argtypes = ...

cf.CFStringGetTypeID.restype = ...
cf.CFStringGetTypeID.argtypes = ...

cf.CFAttributedStringCreate.restype = ...
cf.CFAttributedStringCreate.argtypes = ...

cf.CFURLCreateWithFileSystemPath.restype = ...
cf.CFURLCreateWithFileSystemPath.argtypes = ...

def CFSTR(string): ...
def get_NSString(string): ...
def cfstring_to_string(cfstring): ...

cf.CFDataCreate.restype = ...
cf.CFDataCreate.argtypes = ...

cf.CFDataGetBytes.restype = ...
cf.CFDataGetBytes.argtypes = ...

cf.CFDataGetLength.restype = ...
cf.CFDataGetLength.argtypes = ...

cf.CFDictionaryGetValue.restype = ...
cf.CFDictionaryGetValue.argtypes = ...

cf.CFDictionaryAddValue.restype = ...
cf.CFDictionaryAddValue.argtypes = ...

cf.CFDictionaryCreateMutable.restype = ...
cf.CFDictionaryCreateMutable.argtypes = ...

cf.CFNumberCreate.restype = ...
cf.CFNumberCreate.argtypes = ...

cf.CFNumberGetType.restype = ...
cf.CFNumberGetType.argtypes = ...

cf.CFNumberGetValue.restype = ...
cf.CFNumberGetValue.argtypes = ...

cf.CFNumberGetTypeID.restype = ...
cf.CFNumberGetTypeID.argtypes = ...

cf.CFGetTypeID.restype = ...
cf.CFGetTypeID.argtypes = ...

# CFNumber.h
kCFNumberSInt8Type: int = ...
kCFNumberSInt16Type: int = ...
kCFNumberSInt32Type: int = ...
kCFNumberSInt64Type: int = ...
kCFNumberFloat32Type: int = ...
kCFNumberFloat64Type: int = ...
kCFNumberCharType: int = ...
kCFNumberShortType: int = ...
kCFNumberIntType: int = ...
kCFNumberLongType: int = ...
kCFNumberLongLongType: int = ...
kCFNumberFloatType: int = ...
kCFNumberDoubleType: int = ...
kCFNumberCFIndexType: int = ...
kCFNumberNSIntegerType: int = ...
kCFNumberCGFloatType: int = ...
kCFNumberMaxType: int = ...

def cfnumber_to_number(cfnumber): ...

# Dictionary of cftypes matched to the method converting them to python values.
known_cftypes: dict = ...

def cftype_to_value(cftype): ...

cf.CFSetGetCount.restype = ...
cf.CFSetGetCount.argtypes = ...

cf.CFSetGetValues.restype = ...
# PyPy 1.7 is fine with 2nd arg as POINTER(c_void_p),
# but CPython ctypes 1.1.0 complains, so just use c_void_p.
cf.CFSetGetValues.argtypes = ...

def cfset_to_set(cfset): ...

cf.CFArrayGetCount.restype = ...
cf.CFArrayGetCount.argtypes = ...

cf.CFArrayGetValueAtIndex.restype = ...
cf.CFArrayGetValueAtIndex.argtypes = ...

def cfarray_to_list(cfarray): ...

kCFRunLoopDefaultMode = ...

cf.CFRunLoopGetCurrent.restype = ...
cf.CFRunLoopGetCurrent.argtypes = ...

cf.CFRunLoopGetMain.restype = ...
cf.CFRunLoopGetMain.argtypes = ...

cf.CFShow.restype = ...
cf.CFShow.argtypes = ...

######################################################################

# APPLICATION KIT

# Even though we don't use this directly, it must be loaded so that
# we can find the NSApplication, NSWindow, and NSView classes.
appkit = ...

NSDefaultRunLoopMode = ...
NSEventTrackingRunLoopMode = ...
NSApplicationDidHideNotification = ...
NSApplicationDidUnhideNotification = ...

# /System/Library/Frameworks/AppKit.framework/Headers/NSEvent.h
# NSAnyEventMask = 0xFFFFFFFFL     # NSUIntegerMax
# Commented out b/c not Py3k compatible

NSKeyDown: int = ...
NSKeyUp: int = ...
NSFlagsChanged: int = ...
NSApplicationDefined: int = ...

NSAlphaShiftKeyMask = ...
NSShiftKeyMask = ...
NSControlKeyMask = ...
NSAlternateKeyMask = ...
NSCommandKeyMask = ...
NSNumericPadKeyMask = ...
NSHelpKeyMask = ...
NSFunctionKeyMask = ...

NSInsertFunctionKey: int = ...
NSDeleteFunctionKey: int = ...
NSHomeFunctionKey: int = ...
NSBeginFunctionKey: int = ...
NSEndFunctionKey: int = ...
NSPageUpFunctionKey: int = ...
NSPageDownFunctionKey: int = ...

# /System/Library/Frameworks/AppKit.framework/Headers/NSWindow.h
NSBorderlessWindowMask: int = ...
NSTitledWindowMask = ...
NSClosableWindowMask = ...
NSMiniaturizableWindowMask = ...
NSResizableWindowMask = ...

# /System/Library/Frameworks/AppKit.framework/Headers/NSPanel.h
NSUtilityWindowMask = ...

# /System/Library/Frameworks/AppKit.framework/Headers/NSGraphics.h
NSBackingStoreRetained: int = ...
NSBackingStoreNonretained: int = ...
NSBackingStoreBuffered: int = ...

# /System/Library/Frameworks/AppKit.framework/Headers/NSTrackingArea.h
NSTrackingMouseEnteredAndExited: int = ...
NSTrackingMouseMoved: int = ...
NSTrackingCursorUpdate: int = ...
NSTrackingActiveInActiveApp: int = ...

# /System/Library/Frameworks/AppKit.framework/Headers/NSOpenGL.h
NSOpenGLPFAAllRenderers: int = ...  # choose from all available renderers
NSOpenGLPFADoubleBuffer: int = ...  # choose a double buffered pixel format
NSOpenGLPFAStereo: int = ...  # stereo buffering supported
NSOpenGLPFAAuxBuffers: int = ...  # number of aux buffers
NSOpenGLPFAColorSize: int = ...  # number of color buffer bits
NSOpenGLPFAAlphaSize: int = ...  # number of alpha component bits
NSOpenGLPFADepthSize: int = ...  # number of depth buffer bits
NSOpenGLPFAStencilSize: int = ...  # number of stencil buffer bits
NSOpenGLPFAAccumSize: int = ...  # number of accum buffer bits
NSOpenGLPFAMinimumPolicy: int = ...  # never choose smaller buffers than requested
NSOpenGLPFAMaximumPolicy: int = ...  # choose largest buffers of type requested
NSOpenGLPFAOffScreen: int = ...  # choose an off-screen capable renderer
NSOpenGLPFAFullScreen: int = ...  # choose a full-screen capable renderer
NSOpenGLPFASampleBuffers: int = ...  # number of multi sample buffers
NSOpenGLPFASamples: int = ...  # number of samples per multi sample buffer
NSOpenGLPFAAuxDepthStencil: int = ...  # each aux buffer has its own depth stencil
NSOpenGLPFAColorFloat: int = ...  # color buffers store floating point pixels
NSOpenGLPFAMultisample: int = ...  # choose multisampling
NSOpenGLPFASupersample: int = ...  # choose supersampling
NSOpenGLPFASampleAlpha: int = ...  # request alpha filtering
NSOpenGLPFARendererID: int = ...  # request renderer by ID
NSOpenGLPFASingleRenderer: int = ...  # choose a single renderer for all screens
NSOpenGLPFANoRecovery: int = ...  # disable all failure recovery systems
NSOpenGLPFAAccelerated: int = ...  # choose a hardware accelerated renderer
NSOpenGLPFAClosestPolicy: int = ...  # choose the closest color buffer to request
NSOpenGLPFARobust: int = ...  # renderer does not need failure recovery
NSOpenGLPFABackingStore: int = ...  # back buffer contents are valid after swap
NSOpenGLPFAMPSafe: int = ...  # renderer is multi-processor safe
NSOpenGLPFAWindow: int = ...  # can be used to render to an onscreen window
NSOpenGLPFAMultiScreen: int = ...  # single window can span multiple screens
NSOpenGLPFACompliant: int = ...  # renderer is opengl compliant
NSOpenGLPFAScreenMask: int = ...  # bit mask of supported physical screens
NSOpenGLPFAPixelBuffer: int = ...  # can be used to render to a pbuffer
# can be used to render offline to a pbuffer
NSOpenGLPFARemotePixelBuffer: int = ...
NSOpenGLPFAAllowOfflineRenderers: int = ...  # allow use of offline renderers
# choose a hardware accelerated compute device
NSOpenGLPFAAcceleratedCompute: int = ...
# number of virtual screens in this format
NSOpenGLPFAVirtualScreenCount: int = ...

NSOpenGLCPSwapInterval: int = ...

# /System/Library/Frameworks/ApplicationServices.framework/Frameworks/...
#     CoreGraphics.framework/Headers/CGImage.h
kCGImageAlphaNone: int = ...
kCGImageAlphaPremultipliedLast: int = ...
kCGImageAlphaPremultipliedFirst: int = ...
kCGImageAlphaLast: int = ...
kCGImageAlphaFirst: int = ...
kCGImageAlphaNoneSkipLast: int = ...
kCGImageAlphaNoneSkipFirst: int = ...
kCGImageAlphaOnly: int = ...

kCGImageAlphaPremultipliedLast: int = ...

kCGBitmapAlphaInfoMask: int = ...
kCGBitmapFloatComponents = ...

kCGBitmapByteOrderMask: int = ...
kCGBitmapByteOrderDefault = ...
kCGBitmapByteOrder16Little = ...
kCGBitmapByteOrder32Little = ...
kCGBitmapByteOrder16Big = ...
kCGBitmapByteOrder32Big = ...

# NSApplication.h
NSApplicationPresentationDefault: int = ...
NSApplicationPresentationHideDock = ...
NSApplicationPresentationHideMenuBar = ...
NSApplicationPresentationDisableProcessSwitching = ...
NSApplicationPresentationDisableHideApplication = ...

# NSRunningApplication.h
NSApplicationActivationPolicyRegular: int = ...
NSApplicationActivationPolicyAccessory: int = ...
NSApplicationActivationPolicyProhibited: int = ...

######################################################################

# QUARTZ / COREGRAPHICS

quartz = ...

CGDirectDisplayID = ...  # CGDirectDisplay.h
CGError = ...  # CGError.h
CGBitmapInfo = ...  # CGImage.h

# /System/Library/Frameworks/ApplicationServices.framework/Frameworks/...
#     ImageIO.framework/Headers/CGImageProperties.h
kCGImagePropertyGIFDictionary = ...
kCGImagePropertyGIFDelayTime = ...

# /System/Library/Frameworks/ApplicationServices.framework/Frameworks/...
#     CoreGraphics.framework/Headers/CGColorSpace.h
kCGRenderingIntentDefault: int = ...

quartz.CGDisplayIDToOpenGLDisplayMask.restype = ...
quartz.CGDisplayIDToOpenGLDisplayMask.argtypes = ...

quartz.CGMainDisplayID.restype = ...
quartz.CGMainDisplayID.argtypes = ...

quartz.CGShieldingWindowLevel.restype = ...
quartz.CGShieldingWindowLevel.argtypes = ...

quartz.CGCursorIsVisible.restype = ...

quartz.CGDisplayCopyAllDisplayModes.restype = ...
quartz.CGDisplayCopyAllDisplayModes.argtypes = ...

quartz.CGDisplaySetDisplayMode.restype = ...
quartz.CGDisplaySetDisplayMode.argtypes = ...

quartz.CGDisplayCapture.restype = ...
quartz.CGDisplayCapture.argtypes = ...

quartz.CGDisplayRelease.restype = ...
quartz.CGDisplayRelease.argtypes = ...

quartz.CGDisplayCopyDisplayMode.restype = ...
quartz.CGDisplayCopyDisplayMode.argtypes = ...

quartz.CGDisplayModeGetRefreshRate.restype = ...
quartz.CGDisplayModeGetRefreshRate.argtypes = ...

quartz.CGDisplayModeRetain.restype = ...
quartz.CGDisplayModeRetain.argtypes = ...

quartz.CGDisplayModeRelease.restype = ...
quartz.CGDisplayModeRelease.argtypes = ...

quartz.CGDisplayModeGetWidth.restype = ...
quartz.CGDisplayModeGetWidth.argtypes = ...

quartz.CGDisplayModeGetHeight.restype = ...
quartz.CGDisplayModeGetHeight.argtypes = ...

quartz.CGDisplayModeCopyPixelEncoding.restype = ...
quartz.CGDisplayModeCopyPixelEncoding.argtypes = ...

quartz.CGGetActiveDisplayList.restype = ...
quartz.CGGetActiveDisplayList.argtypes = ...

quartz.CGDisplayBounds.restype = ...
quartz.CGDisplayBounds.argtypes = ...

quartz.CGImageSourceCreateWithData.restype = ...
quartz.CGImageSourceCreateWithData.argtypes = ...

quartz.CGImageSourceCreateImageAtIndex.restype = ...
quartz.CGImageSourceCreateImageAtIndex.argtypes = ...

quartz.CGImageSourceCopyPropertiesAtIndex.restype = ...
quartz.CGImageSourceCopyPropertiesAtIndex.argtypes = ...

quartz.CGImageGetDataProvider.restype = ...
quartz.CGImageGetDataProvider.argtypes = ...

quartz.CGDataProviderCopyData.restype = ...
quartz.CGDataProviderCopyData.argtypes = ...

quartz.CGDataProviderCreateWithCFData.restype = ...
quartz.CGDataProviderCreateWithCFData.argtypes = ...

quartz.CGImageCreate.restype = ...
quartz.CGImageCreate.argtypes = ...

quartz.CGImageRelease.restype = ...
quartz.CGImageRelease.argtypes = ...

quartz.CGImageGetBytesPerRow.restype = ...
quartz.CGImageGetBytesPerRow.argtypes = ...

quartz.CGImageGetWidth.restype = ...
quartz.CGImageGetWidth.argtypes = ...

quartz.CGImageGetHeight.restype = ...
quartz.CGImageGetHeight.argtypes = ...

quartz.CGImageGetBitsPerPixel.restype = ...
quartz.CGImageGetBitsPerPixel.argtypes = ...

quartz.CGImageGetBitmapInfo.restype = ...
quartz.CGImageGetBitmapInfo.argtypes = ...

quartz.CGColorSpaceCreateDeviceRGB.restype = ...
quartz.CGColorSpaceCreateDeviceRGB.argtypes = ...

quartz.CGDataProviderRelease.restype = ...
quartz.CGDataProviderRelease.argtypes = ...

quartz.CGColorSpaceRelease.restype = ...
quartz.CGColorSpaceRelease.argtypes = ...

quartz.CGWarpMouseCursorPosition.restype = ...
quartz.CGWarpMouseCursorPosition.argtypes = ...

quartz.CGDisplayMoveCursorToPoint.restype = ...
quartz.CGDisplayMoveCursorToPoint.argtypes = ...

quartz.CGAssociateMouseAndMouseCursorPosition.restype = ...
quartz.CGAssociateMouseAndMouseCursorPosition.argtypes = ...

quartz.CGBitmapContextCreate.restype = ...
quartz.CGBitmapContextCreate.argtypes = ...

quartz.CGBitmapContextCreateImage.restype = ...
quartz.CGBitmapContextCreateImage.argtypes = ...

quartz.CGFontCreateWithDataProvider.restype = ...
quartz.CGFontCreateWithDataProvider.argtypes = ...

quartz.CGFontCreateWithFontName.restype = ...
quartz.CGFontCreateWithFontName.argtypes = ...

quartz.CGContextDrawImage.restype = ...
quartz.CGContextDrawImage.argtypes = ...

quartz.CGContextRelease.restype = ...
quartz.CGContextRelease.argtypes = ...

quartz.CGContextSetTextPosition.restype = ...
quartz.CGContextSetTextPosition.argtypes = ...

quartz.CGContextSetShouldAntialias.restype = ...
quartz.CGContextSetShouldAntialias.argtypes = ...

quartz.CGDataProviderCreateWithURL.restype = ...
quartz.CGDataProviderCreateWithURL.argtypes = ...

quartz.CGFontCreateWithDataProvider.restype = ...
quartz.CGFontCreateWithDataProvider.argtypes = ...

quartz.CGDisplayScreenSize.argtypes = ...
quartz.CGDisplayScreenSize.restype = ...

quartz.CGDisplayBounds.argtypes = ...
quartz.CGDisplayBounds.restype = ...

######################################################################

# CORETEXT
ct = ...

# Types
CTFontOrientation = ...  # CTFontDescriptor.h
CTFontSymbolicTraits = ...  # CTFontTraits.h

# CoreText constants
kCTFontAttributeName = ...
kCTFontFamilyNameAttribute = ...
kCTFontSymbolicTrait = ...
kCTFontWeightTrait = ...
kCTFontTraitsAttribute = ...

# constants from CTFontTraits.h
kCTFontItalicTrait = ...
kCTFontBoldTrait = ...

ct.CTLineCreateWithAttributedString.restype = ...
ct.CTLineCreateWithAttributedString.argtypes = ...

ct.CTLineDraw.restype = ...
ct.CTLineDraw.argtypes = ...

ct.CTFontGetBoundingRectsForGlyphs.restype = ...
ct.CTFontGetBoundingRectsForGlyphs.argtypes = ...

ct.CTFontGetAdvancesForGlyphs.restype = ...
ct.CTFontGetAdvancesForGlyphs.argtypes = ...

ct.CTFontGetAscent.restype = ...
ct.CTFontGetAscent.argtypes = ...

ct.CTFontGetDescent.restype = ...
ct.CTFontGetDescent.argtypes = ...

ct.CTFontGetSymbolicTraits.restype = ...
ct.CTFontGetSymbolicTraits.argtypes = ...

ct.CTFontGetGlyphsForCharacters.restype = ...
ct.CTFontGetGlyphsForCharacters.argtypes = ...

ct.CTFontCreateWithGraphicsFont.restype = ...
ct.CTFontCreateWithGraphicsFont.argtypes = ...

ct.CTFontCopyFamilyName.restype = ...
ct.CTFontCopyFamilyName.argtypes = ...

ct.CTFontCopyFullName.restype = ...
ct.CTFontCopyFullName.argtypes = ...

ct.CTFontCreateWithFontDescriptor.restype = ...
ct.CTFontCreateWithFontDescriptor.argtypes = ...

ct.CTFontCreateCopyWithAttributes.restype = ...
ct.CTFontCreateCopyWithAttributes.argtypes = ...

ct.CTFontDescriptorCreateWithAttributes.restype = ...
ct.CTFontDescriptorCreateWithAttributes.argtypes = ...

ct.CTTypesetterCreateWithAttributedString.restype = ...
ct.CTTypesetterCreateWithAttributedString.argtypes = ...

ct.CTTypesetterCreateLine.restype = ...
ct.CTTypesetterCreateLine.argtypes = ...

ct.CTLineGetOffsetForStringIndex.restype = ...
ct.CTLineGetOffsetForStringIndex.argtypes = ...

ct.CTFontManagerCreateFontDescriptorsFromURL.restype = ...
ct.CTFontManagerCreateFontDescriptorsFromURL.argtypes = ...

######################################################################

# FOUNDATION

# foundation = cdll.LoadLibrary(util.find_library('Foundation'))

# foundation.NSMouseInRect.restype = c_bool
# foundation.NSMouseInRect.argtypes = [NSPoint, NSRect, c_bool]



from collections import deque

from ...classes.graph import Graph
from ...algorithms.flow.utils import build_residual_network
from ...utils import pairwise

__all__ = ["dinitz"]

def dinitz(
    G: Graph,
    s,
    t,
    capacity: str = "capacity",
    residual=None,
    value_only: bool = False,
    cutoff=None,
): ...
def dinitz_impl(G: Graph, s, t, capacity, residual, cutoff): ...



import datetime
from .cbook import CallbackRegistry
from .transforms import Bbox, Transform
import numpy as np
from typing import Any, Callable, Literal, Type
from .text import Text
from .backend_bases import MouseEvent
from .lines import Line2D
from .axes import Axes
from .artist import Artist, allow_rasterization

from units.basic_units import BasicUnit

from .backend_bases import RendererBase
from datetime import timezone
from .backends.backend_mixed import MixedModeRenderer
from .patches import Patch
from .ticker import (
    Formatter,
    Locator,
)

GRIDLINE_INTERPOLATION_STEPS: int = ...

class Tick(Artist):

    tick1line: Line2D
    tick2line: Line2D
    gridline: Line2D
    label1: Text
    label2: Text

    def __init__(
        self,
        axes,
        loc,
        *,
        size=None,
        width=None,
        color=None,
        tickdir=None,
        pad=None,
        labelsize=None,
        labelcolor=None,
        zorder=None,
        gridOn=None,
        tick1On=True,
        tick2On=True,
        label1On=True,
        label2On=False,
        major=True,
        labelrotation=0,
        grid_color=None,
        grid_linestyle=None,
        grid_linewidth=None,
        grid_alpha=None,
        **kwargs
    ) -> None: ...
    @property
    def label(self)-> Text: ...
    def apply_tickdir(self, tickdir)-> None: ...
    def get_tickdir(self): ...
    def get_tick_padding(self) -> float: ...
    def get_children(self) -> list[Line2D|Text]: ...
    def set_clip_path(
        self, clippath: Patch, transform: Transform | None = None
    ) -> None: ...
    def get_pad_pixels(self)-> float: ...
    def contains(self, mouseevent) -> bool: ...
    def set_pad(self, val: float) -> None: ...
    def get_pad(self) -> float: ...
    def get_loc(self) -> int: ...
    @allow_rasterization
    def draw(self, renderer) -> None: ...
    def set_label1(self, s: str) -> None: ...
    set_label = set_label1
    def set_label2(self, s: str) -> None: ...
    def set_url(self, url: str) -> None: ...
    def get_view_interval(self) -> tuple: ...
    def update_position(self, loc) -> None: ...

class XTick(Tick):
    def __init__(self, *args, **kwargs) -> None: ...
    def update_position(self, loc: int) -> None: ...
    def get_view_interval(self)-> tuple: ...

class YTick(Tick):
    def __init__(self, *args, **kwargs) -> None: ...
    def update_position(self, loc: int) -> None: ...
    def get_view_interval(self)-> tuple: ...

class Ticker:
    def __init__(self) -> None: ...
    @property
    def locator(self)-> Locator: ...
    @locator.setter
    def locator(self, locator: Locator): ...
    @property
    def formatter(self) -> Formatter: ...
    @formatter.setter
    def formatter(self, formatter: Formatter): ...

class _LazyTickList:
    def __init__(self, major: bool) -> None: ...
    def __get__(
        self, instance: XAxis | YAxis, cls: Type[XAxis] | Type[YAxis]
    ) -> list[XTick | YTick]: ...

class Axis(Artist):

    isDefault_label: bool
    axes: Axes
    major: Ticker
    minor: Ticker
    callbacks: CallbackRegistry
    label: Text
    labelpad: float = 4
    offsetText: Text
    pickradius: float

    OFFSETTEXTPAD: int = ...

    def __str__(self) -> str: ...
    def __init__(self, axes: Axes, pickradius: float = ...) -> None: ...
    @property
    def isDefault_majloc(self)-> bool: ...
    @isDefault_majloc.setter
    def isDefault_majloc(self, value): ...
    @property
    def isDefault_majfmt(self)-> bool: ...
    @isDefault_majfmt.setter
    def isDefault_majfmt(self, value): ...
    @property
    def isDefault_minloc(self)-> bool: ...
    @isDefault_minloc.setter
    def isDefault_minloc(self, value): ...
    @property
    def isDefault_minfmt(self)-> bool: ...
    @isDefault_minfmt.setter
    def isDefault_minfmt(self, value): ...

    majorTicks: _LazyTickList = ...
    minorTicks: _LazyTickList = ...
    def get_remove_overlapping_locs(self) -> bool: ...
    def set_remove_overlapping_locs(self, val)-> None: ...

    remove_overlapping_locs: property = ...
    def set_label_coords(self, x: float, y: float, transform: None = ...) -> None: ...
    def get_transform(self)-> Transform: ...
    def get_scale(self) -> str: ...
    def limit_range_for_scale(
        self, vmin: float, vmax: float
    ) -> tuple[float, float]: ...
    def get_children(self)-> list[Text]: ...
    def clear(self) -> None: ...
    def reset_ticks(self) -> None: ...
    def set_tick_params(
        self, which: str = 'major', reset: bool = False, **kwargs
    ) -> None: ...
    def set_clip_path(
        self, clippath: Patch, transform: Transform | None = ...
    ) -> None: ...
    def get_view_interval(self) -> tuple: ...
    def set_view_interval(self, vmin, vmax, ignore: bool = ...)-> None: ...
    def get_data_interval(self) -> tuple: ...
    def set_data_interval(self, vmin, vmax, ignore: bool = ...)-> None: ...
    def get_inverted(self) -> bool: ...
    def set_inverted(self, inverted: bool) -> None: ...
    def set_default_intervals(self)-> None: ...
    def get_ticklabel_extents(self, renderer: RendererBase)-> tuple[Bbox, Bbox]: ...
    def get_tightbbox(
        self, renderer: MixedModeRenderer = ..., *, for_layout_only=...
    )-> Bbox: ...
    def get_tick_padding(self)-> int: ...
    @allow_rasterization
    def draw(self, renderer, *args, **kwargs)-> None: ...
    def get_gridlines(self) -> list[Line2D]: ...
    def get_label(self) -> Text: ...
    def get_offset_text(self) -> Text: ...
    def get_pickradius(self)-> int: ...
    def get_majorticklabels(self) -> list[Text]: ...
    def get_minorticklabels(self) -> list[Text]: ...
    def get_ticklabels(
        self, minor: bool = ..., which: None | Literal["minor", "major", "both"] = ...
    ) -> list[Text]: ...
    def get_majorticklines(self) -> list[Line2D]: ...
    def get_minorticklines(self) -> list[Line2D]: ...
    def get_ticklines(self, minor=...) -> list[Line2D]: ...
    def get_majorticklocs(self) -> list: ...
    def get_minorticklocs(self) -> list: ...
    def get_ticklocs(self, *, minor=...) -> list: ...
    def get_ticks_direction(self, minor: bool = False) -> np.ndarray: ...
    def get_label_text(self) -> str: ...
    def get_major_locator(self) -> Locator: ...
    def get_minor_locator(
        self,
    ) -> Locator: ...
    def get_major_formatter(
        self,
    ) -> Formatter: ...
    def get_minor_formatter(self) -> Formatter: ...
    def get_major_ticks(self, numticks: None | int = ...) -> list[XTick | YTick]: ...
    def get_minor_ticks(self, numticks: None | int = ...) -> list[XTick | YTick]: ...
    def grid(self, visible: bool | None = None, which: str = 'major', **kwargs) -> None: ...
    def update_units(self, data: Any) -> bool: ...
    def have_units(self) -> bool: ...
    def convert_units(self, x): ...
    def set_units(self, u: None | timezone | float | BasicUnit) -> None: ...
    def get_units(self) -> None | timezone | float | BasicUnit: ...
    def set_label_text(self, label: str, fontdict: dict = ..., **kwargs): ...
    def set_major_formatter(self, formatter: Formatter | str | Callable) -> None: ...
    def set_minor_formatter(self, formatter: Formatter | str | Callable) -> None: ...
    def set_major_locator(self, locator: Locator) -> None: ...
    def set_minor_locator(self, locator: Locator) -> None: ...
    def set_pickradius(self, pickradius: float)-> None: ...
    def set_ticklabels(
        self, ticklabels: list[str | Text], *, minor: bool = ..., **kwargs
    ) -> list[Text]: ...
    def set_ticks(
        self,
        ticks: list[float],
        labels: None | list[str] = ...,
        *,
        minor: bool = ...,
        **kwargs
    ) -> list[XTick | YTick]: ...
    def axis_date(self, tz: str | datetime.tzinfo = ...)-> None: ...
    def get_tick_space(self) -> int: ...
    def get_label_position(self) -> str: ...
    def set_label_position(self, position: Literal["top", "bottom"])-> None: ...
    def get_minpos(self): ...

class XAxis(Axis):

    axis_name: str = ...
    _tick_class = XTick
    def __init__(self, *args, **kwargs) -> None: ...
    def contains(self, mouseevent) -> bool: ...
    def set_label_position(self, position: Literal["top", "bottom"]) -> None: ...
    def get_text_heights(self, renderer) -> tuple[float, float]: ...
    def set_ticks_position(
        self, position: Literal["top", "bottom", "both", "default", "none"]
    ) -> None: ...
    def tick_top(self) -> None: ...
    def tick_bottom(self) -> None: ...
    def get_ticks_position(self) -> str: ...
    def get_minpos(self)-> float: ...
    def set_default_intervals(self) -> None: ...
    def get_tick_space(self) -> int: ...

class YAxis(Axis):

    axis_name = ...
    _tick_class = YTick
    def __init__(self, *args, **kwargs) -> None: ...
    def contains(self, mouseevent: MouseEvent) -> bool: ...
    def set_label_position(self, position: Literal["left", "right"]) -> None: ...
    def set_offset_position(self, position: Literal["left", "right"]) -> None: ...
    def get_text_widths(self, renderer): ...
    def set_ticks_position(
        self, position: Literal["left", "right", "both", "default", "none"]
    ) -> None: ...
    def tick_right(self) -> None: ...
    def tick_left(self) -> None: ...
    def get_ticks_position(self) -> str: ...
    def get_minpos(self)-> float: ...
    def set_default_intervals(self) -> None: ...
    def get_tick_space(self) -> int: ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.interpolate._ppoly, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def _croots_poly1() -> typing.Any:
    '\n    Find roots of polynomials.\n\n    This function is for testing croots_poly1\n\n    Parameters\n    ----------\n    c : ndarray, (k, m, n)\n        Coefficients of several order-k polynomials\n    w : ndarray, (k, m, n)\n        Output argument --- roots of the polynomials.\n\n    '
    ...

def evaluate(c, x, xp, dx, extrapolate, out) -> typing.Any:
    '\n    Evaluate a piecewise polynomial.\n\n    Parameters\n    ----------\n    c : ndarray, shape (k, m, n)\n        Coefficients local polynomials of order `k-1` in `m` intervals.\n        There are `n` polynomials in each interval.\n        Coefficient of highest order-term comes first.\n    x : ndarray, shape (m+1,)\n        Breakpoints of polynomials.\n    xp : ndarray, shape (r,)\n        Points to evaluate the piecewise polynomial at.\n    dx : int\n        Order of derivative to evaluate.  The derivative is evaluated\n        piecewise and may have discontinuities.\n    extrapolate : bint\n        Whether to extrapolate to out-of-bounds points based on first\n        and last intervals, or to return NaNs.\n    out : ndarray, shape (r, n)\n        Value of each polynomial at each of the input points.\n        This argument is modified in-place.\n\n    '
    ...

def evaluate_bernstein(c, x, xp, nu, extrapolate, out) -> typing.Any:
    '\n    Evaluate a piecewise polynomial in the Bernstein basis.\n\n    Parameters\n    ----------\n    c : ndarray, shape (k, m, n)\n        Coefficients local polynomials of order `k-1` in `m` intervals.\n        There are `n` polynomials in each interval.\n        Coefficient of highest order-term comes first.\n    x : ndarray, shape (m+1,)\n        Breakpoints of polynomials\n    xp : ndarray, shape (r,)\n        Points to evaluate the piecewise polynomial at.\n    nu : int\n        Order of derivative to evaluate.  The derivative is evaluated\n        piecewise and may have discontinuities.\n    extrapolate : bint, optional\n        Whether to extrapolate to out-of-bounds points based on first\n        and last intervals, or to return NaNs.\n    out : ndarray, shape (r, n)\n        Value of each polynomial at each of the input points.\n        This argument is modified in-place.\n\n    '
    ...

def evaluate_nd(c, xs, ks, xp, dx, extrapolate, out) -> typing.Any:
    '\n    Evaluate a piecewise tensor-product polynomial.\n\n    Parameters\n    ----------\n    c : ndarray, shape (k_1*...*k_d, m_1*...*m_d, n)\n        Coefficients local polynomials of order `k-1` in\n        `m_1`, ..., `m_d` intervals. There are `n` polynomials\n        in each interval.\n    ks : ndarray of int, shape (d,)\n        Orders of polynomials in each dimension\n    xs : d-tuple of ndarray of shape (m_d+1,) each\n        Breakpoints of polynomials\n    xp : ndarray, shape (r, d)\n        Points to evaluate the piecewise polynomial at.\n    dx : ndarray of int, shape (d,)\n        Orders of derivative to evaluate.  The derivative is evaluated\n        piecewise and may have discontinuities.\n    extrapolate : int, optional\n        Whether to extrapolate to out-of-bounds points based on first\n        and last intervals, or to return NaNs.\n    out : ndarray, shape (r, n)\n        Value of each polynomial at each of the input points.\n        For points outside the span ``x[0] ... x[-1]``,\n        ``nan`` is returned.\n        This argument is modified in-place.\n\n    '
    ...

def fix_continuity(c, x, order) -> typing.Any:
    '\n    Make a piecewise polynomial continuously differentiable to given order.\n\n    Parameters\n    ----------\n    c : ndarray, shape (k, m, n)\n        Coefficients local polynomials of order `k-1` in `m` intervals.\n        There are `n` polynomials in each interval.\n        Coefficient of highest order-term comes first.\n\n        Coefficients c[-order-1:] are modified in-place.\n    x : ndarray, shape (m+1,)\n        Breakpoints of polynomials\n    order : int\n        Order up to which enforce piecewise differentiability.\n\n    '
    ...

def integrate(c, x, a, b, extrapolate, out) -> typing.Any:
    '\n    Compute integral over a piecewise polynomial.\n\n    Parameters\n    ----------\n    c : ndarray, shape (k, m, n)\n        Coefficients local polynomials of order `k-1` in `m` intervals.\n    x : ndarray, shape (m+1,)\n        Breakpoints of polynomials\n    a : double\n        Start point of integration.\n    b : double\n        End point of integration.\n    extrapolate : bint, optional\n        Whether to extrapolate to out-of-bounds points based on first\n        and last intervals, or to return NaNs.\n    out : ndarray, shape (n,)\n        Integral of the piecewise polynomial, assuming the polynomial\n        is zero outside the range (x[0], x[-1]).\n        This argument is modified in-place.\n\n    '
    ...

def real_roots() -> typing.Any:
    '\n    Compute real roots of a real-valued piecewise polynomial function.\n\n    If a section of the piecewise polynomial is identically zero, the\n    values (x[begin], nan) are appended to the root list.\n\n    If the piecewise polynomial is not continuous, and the sign\n    changes across a breakpoint, the breakpoint is added to the root\n    set if `report_discont` is True.\n\n    Parameters\n    ----------\n    c, x\n        Polynomial coefficients, as above\n    y : float\n        Find roots of ``pp(x) == y``.\n    report_discont : bint, optional\n        Whether to report discontinuities across zero at breakpoints\n        as roots\n    extrapolate : bint, optional\n        Whether to consider roots obtained by extrapolating based\n        on first and last intervals.\n\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Literal, Sequence
from numpy.random import RandomState
from scipy.sparse._csr import csr_matrix
from scipy import linalg as linalg, sparse as sparse
from numpy import ndarray
from scipy.sparse._dia import dia_matrix
from .sparsefuncs_fast import csr_row_norms as csr_row_norms
from ._array_api import get_namespace as get_namespace
from scipy.sparse import spmatrix
from .validation import check_array as check_array
from .._typing import ArrayLike, Float, MatrixLike, Int
from . import check_random_state as check_random_state

# Authors: Gael Varoquaux
#          Alexandre Gramfort
#          Alexandre T. Passos
#          Olivier Grisel
#          Lars Buitinck
#          Stefan van der Walt
#          Kyle Kastner
#          Giorgio Patrini
# License: BSD 3 clause

import warnings

import numpy as np


def squared_norm(x: ArrayLike) -> Float:
    ...


def row_norms(X: csr_matrix | ArrayLike, squared: bool = False) -> ndarray:
    ...


def fast_logdet(A: MatrixLike) -> Float:
    ...


def density(w: ArrayLike, **kwargs) -> float:
    ...


def safe_sparse_dot(
    a: MatrixLike | ArrayLike | dia_matrix,
    b: MatrixLike | ArrayLike,
    *,
    dense_output: bool = False,
) -> ndarray | spmatrix:
    ...


def randomized_range_finder(
    A: MatrixLike,
    *,
    size: Int,
    n_iter: Int,
    power_iteration_normalizer: Literal["auto", "QR", "LU", "none", "auto"] = "auto",
    random_state: RandomState | None | Int = None,
) -> ndarray:
    ...


def randomized_svd(
    M: MatrixLike | ArrayLike,
    n_components: Int,
    *,
    n_oversamples: Int = 10,
    n_iter: Literal["auto", "auto"] | Int = "auto",
    power_iteration_normalizer: Literal["auto", "QR", "LU", "none", "auto"] = "auto",
    transpose: Literal["auto", "auto"] | bool = "auto",
    flip_sign: bool = True,
    random_state: RandomState | None | Int = None,
    svd_lapack_driver: Literal["gesdd", "gesvd", "gesdd"] = "gesdd",
) -> tuple[ndarray, ndarray, ndarray]:
    ...


def weighted_mode(
    a: ArrayLike, w: ArrayLike, *, axis: Int = 0
) -> tuple[ndarray, ndarray]:
    ...


def cartesian(
    arrays: Sequence[ArrayLike] | ArrayLike, out: None | MatrixLike = None
) -> ndarray:
    ...


def svd_flip(
    u: ArrayLike, v: ArrayLike, u_based_decision: bool = True
) -> tuple[ndarray, ndarray]:
    ...


def log_logistic(
    X: MatrixLike | ArrayLike, out: None | MatrixLike | ArrayLike = None
) -> ndarray:
    ...


def softmax(X: MatrixLike, copy: bool = True) -> ndarray:
    ...


def make_nonnegative(
    X: csr_matrix | ArrayLike, min_value: Float = 0
) -> csr_matrix | ndarray:
    ...


def stable_cumsum(
    arr: ArrayLike, axis: None | Int = None, rtol: Float = 1e-05, atol: Float = 1e-08
) -> ndarray:
    ...



from typing import (
    Any,
    Callable,
    Collection,
    Iterable,
    Optional,
    Sequence,
    Set,
    Type,
    Union,
)

from django.db import IntegrityError
from django.db.models.base import Model
from django.db.models.fields import Field
from django.db.models.options import Options

def CASCADE(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: Sequence[Model],
    using: str,
) -> None: ...
def SET_NULL(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: Sequence[Model],
    using: str,
) -> None: ...
def SET_DEFAULT(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: Sequence[Model],
    using: str,
) -> None: ...
def DO_NOTHING(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: Sequence[Model],
    using: str,
) -> None: ...
def PROTECT(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: Sequence[Model],
    using: str,
) -> None: ...
def RESTRICT(
    collector: "Collector",
    field: Field[Any, Any],
    sub_objs: Sequence[Model],
    using: str,
) -> None: ...
def SET(
    value: Union[Any, Callable[[], Any]]
) -> Callable[["Collector", Field[Any, Any], Sequence[Model], str], None]: ...
def get_candidate_relations_to_delete(
    opts: Options[Any],
) -> Iterable[Field[Any, Any]]: ...

class ProtectedError(IntegrityError):
    protected_objects: Set[Model]
    def __init__(self, msg: str, protected_objects: Set[Model]) -> None: ...

class RestrictedError(IntegrityError):
    restricted_objects: Set[Model]
    def __init__(self, msg: str, restricted_objects: Set[Model]) -> None: ...

class Collector:
    def __init__(self, using: str) -> None: ...
    def collect(
        self,
        objs: Collection[Optional[Model]],
        source: Optional[Type[Model]] = ...,
        source_attr: Optional[str] = ...,
        **kwargs: Any
    ) -> None: ...
    def can_fast_delete(
        self,
        objs: Union[Model, Iterable[Model]],
        from_field: Optional[Field[Any, Any]] = ...,
    ) -> bool: ...



from numpy.typing import NDArray
from itertools import combinations_with_replacement
import itertools
import numpy as np

from skimage._shared import utils
from concurrent.futures import ThreadPoolExecutor

def _texture_filter(gaussian_filtered): ...
def _singlescale_basic_features_singlechannel(
    img, sigma, intensity=True, edges=True, texture=True
): ...
def _mutiscale_basic_features_singlechannel(
    img,
    intensity=True,
    edges=True,
    texture=True,
    sigma_min=0.5,
    sigma_max=16,
    num_sigma=None,
    num_workers=None,
): ...
@utils.deprecate_multichannel_kwarg(multichannel_position=1)
def multiscale_basic_features(
    image: NDArray,
    multichannel: bool = False,
    intensity: bool = True,
    edges: bool = True,
    texture: bool = True,
    sigma_min: float = 0.5,
    sigma_max: float = 16,
    num_sigma: int | None = None,
    num_workers: int | None = None,
    *,
    channel_axis: int | None = None,
): ...



from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
import numpy as np
from ... import glsl
from .collection import Collection
from ..transforms import NullTransform

class RawTriangleCollection(Collection):
    def __init__(self, user_dtype=None, transform=None, vertex=None, fragment=None, **kwargs): ...
    def append(self, points: NDArray, indices: NDArray, **kwargs): ...



"""
This type stub file was generated by pyright.
"""

"""Release data for the IPython project."""
_version_major = ...
_version_minor = ...
_version_patch = ...
_version_extra = ...
_version_extra = ...
_ver = ...
__version__ = ...
if _version_extra:
    __version__ = ...
version = ...
version_info = ...
kernel_protocol_version_info = ...
kernel_protocol_version = ...
license = ...
authors = ...
author = ...
author_email = ...



from vispy.geometry.meshdata import MeshData
from vispy.util.svg.color import Color
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from __future__ import division
from functools import lru_cache

import numpy as np

from .visual import Visual
from .shaders import Function, FunctionChain
from ..gloo import VertexBuffer
from ..geometry import MeshData
from ..color import Color, get_colormap
from ..color.colormap import CubeHelixColormap
from ..util.event import Event

_VERTEX_SHADER: str = ...

_FRAGMENT_SHADER: str = ...

class MeshVisual(Visual):

    _shaders: dict = ...

    def __init__(
        self,
        vertices: ArrayLike | None = None,
        faces: ArrayLike | None = None,
        vertex_colors: ArrayLike | None = None,
        face_colors: ArrayLike | None = None,
        color: Color = ...,
        vertex_values: ArrayLike | None = None,
        meshdata: None | MeshData = None,
        shading: str | None = None,
        mode: str = "triangles",
        **kwargs,
    ): ...
    @property
    def shading(self): ...
    @shading.setter
    def shading(self, shading): ...
    def set_data(
        self,
        vertices: ArrayLike | None = None,
        faces: ArrayLike | None = None,
        vertex_colors: ArrayLike | None = None,
        face_colors: ArrayLike | None = None,
        color: Color | None = None,
        vertex_values: ArrayLike | None = None,
        meshdata: None | MeshData = None,
    ): ...
    @property
    def clim(self): ...
    @clim.setter
    def clim(self, clim): ...
    @property
    def _clim_values(self): ...
    @property
    def cmap(self): ...
    @cmap.setter
    def cmap(self, cmap): ...
    @property
    def mode(self): ...
    @mode.setter
    def mode(self, m): ...
    @property
    def mesh_data(self): ...
    @property
    def color(self): ...
    @color.setter
    def color(self, c): ...
    def mesh_data_changed(self): ...
    def _build_color_transform(self, colors): ...
    @staticmethod
    @lru_cache(maxsize=2)
    def _ensure_vec4_func(dims): ...
    def _update_data(self): ...
    def _prepare_draw(self, view): ...
    @staticmethod
    def _prepare_transforms(view): ...
    def _compute_bounds(self, axis, view): ...



import numpy as np
from pandas.core.frame import DataFrame

def unstack(obj, level, fill_value=...): ...
def stack(frame, level: int = ..., dropna: bool = ...): ...
def stack_multiple(frame, level, dropna: bool = ...): ...
def get_dummies(
    data,
    prefix=...,
    prefix_sep=...,
    dummy_na=...,
    columns=...,
    sparse=...,
    drop_first=...,
    dtype=...,
) -> DataFrame: ...



from vispy.util.svg.color import Color
from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from ..geometry import create_plane
from .visual import CompoundVisual
from .mesh import MeshVisual

class PlaneVisual(CompoundVisual):
    def __init__(
        self,
        width: float = 1,
        height: float = 1,
        width_segments: int = 1,
        height_segments: float = 1,
        direction: str = "+z",
        vertex_colors: NDArray | None = None,
        face_colors: NDArray | None = None,
        color: Color = ...,
        edge_color: tuple | Color | None = None,
    ): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.integrate._test_odeint_banded, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def banded5x5(t, y, f, n=...) -> typing.Any:
    "banded5x5(t,y,f,[n])\n\nWrapper for ``banded5x5``.\n\nParameters\n----------\nt : input float\ny : input rank-1 array('d') with bounds (n)\nf : input rank-1 array('d') with bounds (n)\n\nOther Parameters\n----------------\nn : input int, optional\n    Default: len(y)\n"
    ...

def banded5x5_bjac(t, y, ml, mu, bjac, n=..., nrowpd=...) -> typing.Any:
    "banded5x5_bjac(t,y,ml,mu,bjac,[n,nrowpd])\n\nWrapper for ``banded5x5_bjac``.\n\nParameters\n----------\nt : input float\ny : input rank-1 array('d') with bounds (5)\nml : input int\nmu : input int\nbjac : input rank-2 array('d') with bounds (nrowpd,n)\n\nOther Parameters\n----------------\nn : input int, optional\n    Default: shape(bjac,1)\nnrowpd : input int, optional\n    Default: shape(bjac,0)\n"
    ...

def banded5x5_jac(t, y, ml, mu, jac, n=..., nrowpd=...) -> typing.Any:
    "banded5x5_jac(t,y,ml,mu,jac,[n,nrowpd])\n\nWrapper for ``banded5x5_jac``.\n\nParameters\n----------\nt : input float\ny : input rank-1 array('d') with bounds (n)\nml : input int\nmu : input int\njac : input rank-2 array('d') with bounds (nrowpd,n)\n\nOther Parameters\n----------------\nn : input int, optional\n    Default: len(y)\nnrowpd : input int, optional\n    Default: shape(jac,0)\n"
    ...

def banded5x5_solve(y, nsteps, dt, jt) -> typing.Any:
    "nst,nfe,nje = banded5x5_solve(y,nsteps,dt,jt)\n\nWrapper for ``banded5x5_solve``.\n\nParameters\n----------\ny : in/output rank-1 array('d') with bounds (5)\nnsteps : input int\ndt : input float\njt : input int\n\nReturns\n-------\nnst : int\nnfe : int\nnje : int\n"
    ...

def getbands() -> typing.Any:
    "jac = getbands()\n\nWrapper for ``getbands``.\n\nReturns\n-------\njac : rank-2 array('d') with bounds (4,5)\n"
    ...

def jac() -> typing.Any:
    "'d'-array(4,5)\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from ...classes.graph import Graph

__all__ = ["build_auxiliary_node_connectivity", "build_auxiliary_edge_connectivity"]

def build_auxiliary_node_connectivity(G: Graph): ...
def build_auxiliary_edge_connectivity(G: Graph): ...



from typing import Any, Optional

from django.contrib.admin.options import ModelAdmin
from django.core.handlers.wsgi import WSGIRequest
from django.db.models.query import QuerySet
from django.template.response import TemplateResponse

def delete_selected(
    modeladmin: ModelAdmin[Any], request: WSGIRequest, queryset: QuerySet[Any]
) -> Optional[TemplateResponse]: ...



from ._typing import *
from .colors import Colormap, Normalize

from collections.abc import Mapping

class __getattr__:
    LUTSIZE = ...

class ColormapRegistry(Mapping):
    def __init__(self, cmaps) -> None: ...
    def __getitem__(self, item: str): ...
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def __str__(self) -> str: ...
    def __call__(self): ...
    def register(
        self, cmap: Colormap, *, name: str = ..., force: bool = ...
    ) -> None: ...
    def unregister(self, name: str) -> None: ...

def register_cmap(
    name: str = ..., cmap: Colormap = ..., *, override_builtin: bool = ...
) -> None: ...
def get_cmap(name: Colormap | str | None = ..., lut: int | None = ...) -> Colormap: ...
def unregister_cmap(name: str) -> Colormap | None: ...

class ScalarMappable:
    def __init__(
        self, norm: Normalize | None = ..., cmap: str | Colormap = ...
    ) -> None: ...
    callbacksSM = ...
    def to_rgba(self, x, alpha=..., bytes=..., norm=...)-> tuple[float, float, float, float]: ...
    def set_array(self, A: ArrayLike | None) -> None: ...
    def get_array(self): ...
    def get_cmap(self) -> Colormap: ...
    def get_clim(self) -> tuple: ...
    def set_clim(self, vmin: float = ..., vmax: float = ...)->None: ...
    def get_alpha(self) -> float: ...
    def set_cmap(self, cmap: Colormap | str | None)-> None: ...
    @property
    def norm(self) -> Normalize: ...
    @norm.setter
    def norm(self, norm: Normalize)-> None: ...
    def set_norm(self, norm: Normalize | None): ...
    def autoscale(self) -> None: ...
    def autoscale_None(self) -> None: ...
    def changed(self) -> None: ...



from vispy.util.svg.color import Color
from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import warnings
from .box import BoxVisual

class CubeVisual(BoxVisual):
    def __init__(
        self,
        size: float | tuple = 1.0,
        vertex_colors: NDArray | None = None,
        face_colors: NDArray | None = None,
        color: Color = ...,
        edge_color: tuple | Color | None = None,
        **kwargs,
    ): ...



from typing import Any, Iterable, Iterator, Sequence, SupportsIndex
from numpy.random import RandomState
from .fixes import parse_version as parse_version, threadpool_info as threadpool_info
from ._bunch import Bunch as Bunch
from .murmurhash import murmurhash3_32 as murmurhash3_32
from itertools import compress as compress, islice as islice
from ..exceptions import DataConversionWarning as DataConversionWarning
from .validation import (
    as_float_array as as_float_array,
    assert_all_finite as assert_all_finite,
    check_random_state as check_random_state,
    column_or_1d as column_or_1d,
    check_array as check_array,
    check_consistent_length as check_consistent_length,
    check_X_y as check_X_y,
    indexable as indexable,
    check_symmetric as check_symmetric,
    check_scalar as check_scalar,
)
from contextlib import contextmanager as contextmanager, suppress as suppress
from .discovery import all_estimators as all_estimators
from scipy.sparse import issparse as issparse
from .. import get_config as get_config
from collections.abc import Sequence as Sequence
from .deprecation import deprecated as deprecated
from numpy import ndarray
from .class_weight import (
    compute_class_weight as compute_class_weight,
    compute_sample_weight as compute_sample_weight,
)
from ._estimator_html_repr import estimator_html_repr as estimator_html_repr
from .._typing import MatrixLike, ArrayLike, Int
import math as math
import numbers as numbers
import platform as platform
import struct as struct
import timeit as timeit

import warnings as warnings
import numpy as np


# Do not deprecate parallel_backend and register_parallel_backend as they are
# needed to tune `scikit-learn` behavior and have different effect if called
# from the vendored version or or the site-package version. The other are
# utilities that are independent of scikit-learn so they are not part of
# scikit-learn public API.
parallel_backend = ...
register_parallel_backend = ...

__all__ = [
    "murmurhash3_32",
    "as_float_array",
    "assert_all_finite",
    "check_array",
    "check_random_state",
    "compute_class_weight",
    "compute_sample_weight",
    "column_or_1d",
    "check_consistent_length",
    "check_X_y",
    "check_scalar",
    "indexable",
    "check_symmetric",
    "indices_to_mask",
    "deprecated",
    "parallel_backend",
    "register_parallel_backend",
    "resample",
    "shuffle",
    "check_matplotlib_support",
    "all_estimators",
    "DataConversionWarning",
    "estimator_html_repr",
    "Bunch",
]

IS_PYPY = ...
_IS_32BIT = ...


def safe_mask(X: MatrixLike | ArrayLike, mask: ArrayLike) -> ndarray:
    ...


def axis0_safe_slice(
    X: MatrixLike | ArrayLike, mask: ArrayLike, len_mask: Int
) -> ndarray:
    ...


def resample(
    *arrays,
    replace: bool = True,
    n_samples: None | Int = None,
    random_state: RandomState | None | Int = None,
    stratify: None | MatrixLike | ArrayLike = None
) -> list[ndarray]:
    ...


def shuffle(
    *arrays, random_state: RandomState | None | Int = None, n_samples: None | Int = None
) -> list[SupportsIndex]:
    ...


def safe_sqr(X: MatrixLike | ArrayLike, *, copy: bool = True) -> ndarray:
    ...


def gen_batches(n: Int, batch_size: Int, *, min_batch_size: Int = 0) -> Iterator[slice]:
    ...


def gen_even_slices(
    n: Int, n_packs: Int, *, n_samples: None | Int = None
) -> Iterator[slice]:
    ...


def tosequence(x: Iterable) -> list:
    ...


def indices_to_mask(indices: Sequence | ndarray, mask_length: Int) -> ndarray:
    ...


def get_chunk_n_rows(
    row_bytes: Int,
    *,
    max_n_rows: None | Int = None,
    working_memory: float | None | int = None
) -> int:
    ...


def is_scalar_nan(x: Any) -> bool:
    ...


def check_matplotlib_support(caller_name: str) -> None:
    ...


def check_pandas_support(caller_name: str):
    ...



from matplotlib.transforms import Bbox
import numpy as np
from matplotlib._typing import *
from matplotlib.text import Text
from matplotlib.font_manager import FontProperties
from io import BufferedWriter, BytesIO
from matplotlib.font_manager import FontProperties
from typing import Any, Tuple

import threading
from matplotlib.backend_bases import (
    FigureCanvasBase,
    FigureManagerBase,
    GraphicsContextBase,
    RendererBase,
    _Backend,
)

backend_version: str = ...

def get_hinting_flag() -> int: ...

class RendererAgg(RendererBase):

    lock: threading.RLock = ...

    def __init__(self, width, height, dpi) -> None: ...
    def __getstate__(self) -> dict[str, Any]: ...
    def __setstate__(self, state)-> None: ...
    def draw_path(self, gc, path, transform, rgbFace=...)-> None: ...
    def draw_mathtext(self, gc, x, y, s, prop, angle)-> None: ...
    def draw_text(
        self,
        gc: GraphicsContextBase,
        x: float,
        y: float,
        s: str,
        prop: FontProperties,
        angle: float,
        ismath=...,
        mtext: Text = ...,
    )-> None: ...
    def get_text_width_height_descent(
        self, s: str, prop: FontProperties, ismath: bool
    ) -> tuple[float, float, float]: ...
    def draw_tex(self, gc: GraphicsContextBase, x, y, s: str, prop: FontProperties, angle, *, mtext=...)-> None: ...
    def get_canvas_width_height(self)-> tuple[float, float]: ...
    def points_to_pixels(self, points: int|float) -> float: ...
    def buffer_rgba(self) -> memoryview: ...
    def tostring_argb(self) -> bytes: ...
    def tostring_rgb(self) -> bytes: ...
    def clear(self) -> None: ...
    def option_image_nocomposite(self) -> bool: ...
    def option_scale_image(self) -> bool: ...
    def restore_region(self, region, bbox=..., xy=...) -> None: ...
    def start_filter(self) -> None: ...
    def stop_filter(self, post_processing)-> None: ...

class FigureCanvasAgg(FigureCanvasBase):
    def copy_from_bbox(self, bbox: Bbox): ...
    def restore_region(self, region, bbox: Bbox=..., xy=...): ...
    def draw(self) -> None: ...
    def get_renderer(self, cleared: bool = ...) -> RendererAgg: ...
    def tostring_rgb(self) -> str: ...
    def tostring_argb(self) -> np.str_: ...
    def buffer_rgba(self) -> memoryview: ...
    def print_raw(self, filename_or_obj) -> None: ...

    print_rgba = print_raw

    def print_png(
        self,
        filename_or_obj: BytesIO|BufferedWriter|str,
        metadata: dict = ...,
        pil_kwargs: dict = ...
    ) -> None: ...
    def print_to_buffer(self)-> tuple[bytes, tuple[int, int]]: ...
    def print_jpg(
        self, filename_or_obj: str | PathLike | FileLike, pil_kwargs: dict = ...
    )-> None: ...

    print_jpeg = print_jpg
    def print_tif(
        self, filename_or_obj: str | PathLike | FileLike, *, pil_kwargs: dict = ...
    )-> None: ...

    print_tiff = print_tif
    def print_webp(
        self, filename_or_obj: str | PathLike | FileLike, *, pil_kwargs: dict = ...
    )-> None: ...

class _BackendAgg(_Backend):
    FigureCanvas = FigureCanvasAgg
    FigureManager = FigureManagerBase



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

from ..shaders import Function
from ...util.event import EventEmitter

class BaseTransform(object):

    glsl_map: None = ...  # Must be GLSL code
    glsl_imap: None = ...

    # Flags used to describe the transformation. Subclasses should define each
    # as True or False.
    # (usually used for making optimization decisions)

    # If True, then for any 3 colinear points, the
    # transformed points will also be colinear.
    Linear: None = ...

    # The transformation's effect on one axis is independent
    # of the input position along any other axis.
    Orthogonal: None = ...

    # If True, then the distance between two points is the
    # same as the distance between the transformed points.
    NonScaling: None = ...

    # Scale factors are applied equally to all axes.
    Isometric: None = ...

    def __init__(self): ...
    def map(self, obj: tuple[float, float] | tuple[float, float, float]): ...
    def imap(self, obj: tuple[float, float] | tuple[float, float, float]): ...
    @property
    def inverse(self): ...
    @property
    def dynamic(self): ...
    @dynamic.setter
    def dynamic(self, d): ...
    def shader_map(self): ...
    def shader_imap(self): ...
    def _shader_object(self): ...
    def update(self, *args): ...
    def __mul__(self, tr): ...
    def __rmul__(self, tr): ...
    def __repr__(self): ...
    def __del__(self): ...

class InverseTransform(BaseTransform):
    def __init__(self, transform): ...
    @property
    def Linear(self): ...
    @property
    def Orthogonal(self): ...
    @property
    def NonScaling(self): ...
    @property
    def Isometric(self): ...
    @property
    def shader_map(self): ...
    @property
    def shader_imap(self): ...
    def __repr__(self): ...

# import here to avoid import cycle; needed for BaseTransform.__mul__.
from .chain import ChainTransform  # noqa



from typing import Any, Container, Dict, Iterator, List, Set

class CyclicDependencyError(ValueError): ...

def topological_sort_as_sets(
    dependency_graph: Dict[Any, Any]
) -> Iterator[Set[Any]]: ...
def stable_topological_sort(
    l: Container[Any], dependency_graph: Dict[Any, Any]  # noqa:E741
) -> List[Any]: ...



from numpy import ndarray
from vispy.util.svg.color import Color
from numpy.typing import NDArray
from vispy._typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from .line import LineVisual
from ..color import ColorArray
from ..color.colormap import _normalize, get_colormap

def iso_mesh_line(
    vertices: NDArray, tris: NDArray, vertex_data: NDArray, levels: NDArray
) -> tuple[np.ndarray, np.ndarray, np.ndarray]: ...

class IsolineVisual(LineVisual):
    def __init__(
        self,
        vertices: NDArray | None = None,
        tris: NDArray | None = None,
        data: NDArray | None = None,
        levels: NDArray | None = None,
        color_lev: Color | tuple | ArrayLike | str | None = None,
        **kwargs,
    ): ...
    @property
    def levels(self): ...
    @levels.setter
    def levels(self, levels): ...
    @property
    def data(self): ...
    def set_data(
        self,
        vertices: NDArray | None = None,
        tris: NDArray | None = None,
        data: NDArray | None = None,
    ): ...
    @property
    def color(self): ...
    def set_color(self, color: Color): ...
    def _levels_to_colors(self): ...
    def _compute_iso_color(self): ...
    def _prepare_draw(self, view): ...



from ._bounds import l1_min_c as l1_min_c
from ._classes import (
    SVC as SVC,
    NuSVC as NuSVC,
    SVR as SVR,
    NuSVR as NuSVR,
    OneClassSVM as OneClassSVM,
    LinearSVC as LinearSVC,
    LinearSVR as LinearSVR,
)

__all__ = [
    "LinearSVC",
    "LinearSVR",
    "NuSVC",
    "NuSVR",
    "OneClassSVM",
    "SVC",
    "SVR",
    "l1_min_c",
]



from skimage.measure._regionprops import RegionProperties
from numpy.typing import ArrayLike
from typing import Mapping
import inspect
from functools import wraps
from math import atan2
from math import pi as PI
from math import sqrt
from warnings import warn

import numpy as np
from scipy import ndimage as ndi
from scipy.spatial.distance import pdist

from . import _moments
from ._find_contours import find_contours
from ._marching_cubes_lewiner import marching_cubes
from ._regionprops_utils import euler_number, perimeter, perimeter_crofton

__all__ = ["regionprops", "euler_number", "perimeter", "perimeter_crofton"]

# All values in this PROPS dict correspond to current scikit-image property
# names. The keys in this PROPS dict correspond to older names used in prior
# releases. For backwards compatibility, these older names will continue to
# work, but will not be documented.
PROPS: dict = ...

OBJECT_COLUMNS: set = ...

COL_DTYPES: dict = ...

PROP_VALS = ...

def _infer_number_of_required_args(func): ...
def _infer_regionprop_dtype(func, *, intensity, ndim): ...
def _cached(f): ...
def only2d(method): ...
def _inertia_eigvals_to_axes_lengths_3D(inertia_tensor_eigvals): ...

class RegionProperties:
    def __init__(
        self,
        slice,
        label,
        label_image,
        intensity_image,
        cache_active,
        *,
        extra_properties=None
    ): ...
    def __getattr__(self, attr): ...
    def __setattr__(self, name, value): ...
    @property
    @_cached
    def area(self): ...
    @property
    def bbox(self): ...
    @property
    def area_bbox(self): ...
    @property
    def centroid(self): ...
    @property
    @_cached
    def area_convex(self): ...
    @property
    @_cached
    def image_convex(self): ...
    @property
    def coords(self): ...
    @property
    @only2d
    def eccentricity(self): ...
    @property
    def equivalent_diameter_area(self): ...
    @property
    def euler_number(self): ...
    @property
    def extent(self): ...
    @property
    def feret_diameter_max(self): ...
    @property
    def area_filled(self): ...
    @property
    @_cached
    def image_filled(self): ...
    @property
    @_cached
    def image(self): ...
    @property
    @_cached
    def inertia_tensor(self): ...
    @property
    @_cached
    def inertia_tensor_eigvals(self): ...
    @property
    @_cached
    def image_intensity(self): ...
    def _image_intensity_double(self): ...
    @property
    def centroid_local(self): ...
    @property
    def intensity_max(self): ...
    @property
    def intensity_mean(self): ...
    @property
    def intensity_min(self): ...
    @property
    def axis_major_length(self): ...
    @property
    def axis_minor_length(self): ...
    @property
    @_cached
    def moments(self): ...
    @property
    @_cached
    def moments_central(self): ...
    @property
    @only2d
    def moments_hu(self): ...
    @property
    @_cached
    def moments_normalized(self): ...
    @property
    @only2d
    def orientation(self): ...
    @property
    @only2d
    def perimeter(self): ...
    @property
    @only2d
    def perimeter_crofton(self): ...
    @property
    def solidity(self): ...
    @property
    def centroid_weighted(self): ...
    @property
    def centroid_weighted_local(self): ...
    @property
    @_cached
    def moments_weighted(self): ...
    @property
    @_cached
    def moments_weighted_central(self): ...
    @property
    @only2d
    def moments_weighted_hu(self): ...
    @property
    @_cached
    def moments_weighted_normalized(self): ...
    def __iter__(self): ...
    def __getitem__(self, key): ...
    def __eq__(self, other): ...

# For compatibility with code written prior to 0.16
_RegionProperties = RegionProperties

def _props_to_dict(regions, properties=("label", "bbox"), separator="-"): ...
def regionprops_table(
    label_image,
    intensity_image=None,
    properties: tuple | ArrayLike = ...,
    *,
    cache: bool = True,
    separator: str = "-",
    extra_properties=None
) -> Mapping: ...
def regionprops(
    label_image,
    intensity_image=None,
    cache: bool = True,
    coordinates=None,
    *,
    extra_properties=None
) -> list[RegionProperties]: ...
def _parse_docs(): ...
def _install_properties_docs(): ...



import contextlib
from typing import Generator

__all__ = ["use", "context", "available", "library", "reload_library"]

class __getattr__:
    STYLE_FILE_PATTERN = ...

BASE_LIBRARY_PATH: str = ...
USER_LIBRARY_PATHS: list[str] = ...
STYLE_EXTENSION: str = ...
STYLE_BLACKLIST: set[str] = ...

def use(style: str) -> None: ...
@contextlib.contextmanager
def context(style: str, after_reset: bool = ...)-> Generator: ...
def load_base_library() -> dict: ...
def iter_user_libraries()-> Generator: ...
def update_user_library(library): ...
def read_style_directory(
    style_dir: str,
) -> dict: ...
def update_nested_dict(main_dict: dict, new_dict: dict): ...

library = ...
available: list = ...

def reload_library() -> None: ...



from . import backend_agg, backend_gtk3
from .backend_gtk3 import _BackendGTK3

class FigureCanvasGTK3Agg(backend_gtk3.FigureCanvasGTK3, backend_agg.FigureCanvasAgg):
    def __init__(self, figure) -> None: ...
    def on_draw_event(self, widget, ctx) -> bool: ...
    def blit(self, bbox=...)-> None: ...
    def draw(self)-> None: ...

class FigureManagerGTK3Agg(backend_gtk3.FigureManagerGTK3): ...

class _BackendGTK3Cairo(_BackendGTK3):
    FigureCanvas = FigureCanvasGTK3Agg



from typing import Any, Dict, Optional, Tuple, Type, Union

from django.contrib.auth.models import Permission
from django.db import models
from django.db.models import Manager
from django.db.models.base import Model
from django.db.models.manager import BaseManager
from django.db.models.query import QuerySet

class ContentTypeManager(BaseManager["ContentType"]):
    def get_by_natural_key(self, app_label: str, model: str) -> ContentType: ...
    def get_for_model(
        self, model: Union[Type[Model], Model], for_concrete_model: bool = ...
    ) -> ContentType: ...
    def get_for_models(
        self, *models: Any, for_concrete_models: bool = ...
    ) -> Dict[Type[Model], ContentType]: ...
    def get_for_id(self, id: int) -> ContentType: ...
    def clear_cache(self) -> None: ...

class ContentType(models.Model):
    id: int
    app_label: models.CharField[Any] = ...
    model: models.CharField[Any] = ...
    objects: ContentTypeManager = ...
    permission_set: Manager[Permission]
    @property
    def name(self) -> str: ...
    def model_class(self) -> Optional[Type[Model]]: ...
    def get_object_for_this_type(self, **kwargs: Any) -> Model: ...
    def get_all_objects_for_this_type(self, **kwargs: Any) -> QuerySet[Any]: ...
    def natural_key(self) -> Tuple[str, str]: ...





# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import annotations

from typing import Optional
from functools import lru_cache

from ._scalable_textures import CPUScaledTexture3D, GPUScaledTextured3D, Texture2D
from ..gloo import VertexBuffer, IndexBuffer
from ..gloo.texture import should_cast_to_f32
from . import Visual
from .shaders import Function
from ..color import get_colormap
from ..io import load_spatial_filters

import numpy as np
from typing import Literal
from numpy.typing import NDArray, ArrayLike

# todo: implement more render methods (port from visvis)
# todo: allow anisotropic data
# todo: what to do about lighting? ambi/diffuse/spec/shinynes on each visual?

_VERTEX_SHADER: str = ...  # noqa

_FRAGMENT_SHADER: str = ...  # noqa

_RAYCASTING_SETUP_VOLUME: str = ...

_RAYCASTING_SETUP_PLANE: str = ...

_MIP_SNIPPETS = ...

_ATTENUATED_MIP_SNIPPETS = ...

_MINIP_SNIPPETS = ...

_TRANSLUCENT_SNIPPETS = ...

_ADDITIVE_SNIPPETS = ...

_ISO_SNIPPETS = ...

_AVG_SNIPPETS = ...

_INTERPOLATION_TEMPLATE: str = ...

_TEXTURE_LOOKUP: str = ...

class VolumeVisual(Visual):

    _rendering_methods: dict = ...

    _raycasting_modes: dict = ...

    _shaders: dict = ...

    _func_templates: dict = ...

    def __init__(
        self,
        vol: NDArray,
        clim: str | tuple = "auto",
        method: Literal["mip", "attenuated_mip", "minip", "translucent", "additive"] = "mip",
        threshold: float | None = None,
        attenuation: float = 1.0,
        relative_step_size: float = 0.8,
        cmap: str = "grays",
        gamma: float = 1.0,
        interpolation: str = "linear",
        texture_format: np.dtype | None | str = None,
        raycasting_mode: Literal["volume", "plane"] = "volume",
        plane_position: ArrayLike | None = None,
        plane_normal: ArrayLike | None = None,
        plane_thickness: float = 1.0,
        clipping_planes=None,
        clipping_planes_coord_system="scene",
        mip_cutoff=None,
        minip_cutoff=None,
    ): ...
    def _init_interpolation(self, interpolation_methods): ...
    def _create_texture(self, texture_format, data): ...
    def set_data(self, vol: NDArray, clim: tuple | None = None, copy: bool = True): ...
    @property
    def rendering_methods(self): ...
    @property
    def raycasting_modes(self): ...
    @property
    def clim(self): ...
    @clim.setter
    def clim(self, value): ...
    @property
    def gamma(self): ...
    @gamma.setter
    def gamma(self, value): ...
    @property
    def cmap(self): ...
    @cmap.setter
    def cmap(self, cmap): ...
    @property
    def interpolation_methods(self): ...
    @property
    def interpolation(self): ...
    @interpolation.setter
    def interpolation(self, i): ...

    # The interpolation code could be transferred to a dedicated filter
    # function in visuals/filters as discussed in #1051
    def _build_interpolation(self): ...
    @staticmethod
    @lru_cache(maxsize=10)
    def _build_clipping_planes_glsl(n_planes: int) -> str: ...
    @property
    def clipping_planes(self) -> np.ndarray: ...
    @clipping_planes.setter
    def clipping_planes(self, value: Optional[np.ndarray]): ...
    @property
    def clipping_planes_coord_system(self) -> str: ...
    @property
    def _before_loop_snippet(self): ...
    @property
    def _in_loop_snippet(self): ...
    @property
    def _after_loop_snippet(self): ...
    @property
    def method(self): ...
    @method.setter
    def method(self, method): ...
    @property
    def _raycasting_setup_snippet(self): ...
    @property
    def raycasting_mode(self): ...
    @raycasting_mode.setter
    def raycasting_mode(self, value: str): ...
    @property
    def threshold(self): ...
    @threshold.setter
    def threshold(self, value): ...
    @property
    def attenuation(self): ...
    @attenuation.setter
    def attenuation(self, value): ...
    @property
    def relative_step_size(self): ...
    @relative_step_size.setter
    def relative_step_size(self, value): ...
    @property
    def plane_position(self): ...
    @plane_position.setter
    def plane_position(self, value): ...
    @property
    def plane_normal(self): ...
    @plane_normal.setter
    def plane_normal(self, value): ...
    @property
    def plane_thickness(self): ...
    @plane_thickness.setter
    def plane_thickness(self, value: float): ...
    @property
    def mip_cutoff(self): ...
    @mip_cutoff.setter
    def mip_cutoff(self, value): ...
    @property
    def minip_cutoff(self): ...
    @minip_cutoff.setter
    def minip_cutoff(self, value): ...
    def _create_vertex_data(self): ...
    def _compute_bounds(self, axis, view): ...
    def _prepare_transforms(self, view): ...
    def _prepare_draw(self, view): ...



from typing import Literal
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np

from .image import ImageVisual
from ..util.fourier import stft, fft_freqs

class SpectrogramVisual(ImageVisual):
    def __init__(
        self,
        x: ArrayLike | None = None,
        n_fft: int = 256,
        step: None | int = None,
        fs: float = 1.0,
        window: str | None = "hann",
        normalize: bool = False,
        color_scale: Literal["linear", "log"] = "log",
        cmap: str = "cubehelix",
        clim: str | tuple = "auto",
    ): ...
    @property
    def freqs(self): ...
    @property
    def x(self): ...
    @x.setter
    def x(self, x): ...
    @property
    def n_fft(self): ...
    @n_fft.setter
    def n_fft(self, n_fft): ...
    @property
    def step(self): ...
    @step.setter
    def step(self, step): ...
    @property
    def fs(self): ...
    @fs.setter
    def fs(self, fs): ...
    @property
    def window(self): ...
    @window.setter
    def window(self, window): ...
    @property
    def color_scale(self): ...
    @color_scale.setter
    def color_scale(self, color_scale): ...
    @property
    def normalize(self): ...
    @normalize.setter
    def normalize(self, normalize): ...
    def _calculate_spectrogram(self): ...
    def _update_image(self): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from . import BaseGLProxy, _copy_gl_functions
from ._constants import *  # noqa

class DummyProxy(BaseGLProxy):
    def __call__(self, funcname, returns, *args): ...

# Instantiate proxy and inject functions
_proxy = ...



from typing import Mapping

from ...algorithms.components import connected_components
from ...classes.graph import Graph
from ...exception import AmbiguousSolution

__all__ = [
    "is_bipartite",
    "is_bipartite_node_set",
    "color",
    "sets",
    "density",
    "degrees",
]

def color(G: Graph) -> Mapping: ...
def is_bipartite(G: Graph): ...
def is_bipartite_node_set(G: Graph, nodes): ...
def sets(G: Graph, top_nodes=None) -> tuple[set, set]: ...
def density(B, nodes) -> float: ...
def degrees(B, nodes, weight=None) -> tuple[dict, ...]: ...



import warnings
from collections.abc import Mapping

__all__ = [
    "AtlasView",
    "AdjacencyView",
    "MultiAdjacencyView",
    "UnionAtlas",
    "UnionAdjacency",
    "UnionMultiInner",
    "UnionMultiAdjacency",
    "FilterAtlas",
    "FilterAdjacency",
    "FilterMultiInner",
    "FilterMultiAdjacency",
]

class AtlasView(Mapping):

    __slots__ = ...

    def __getstate__(self): ...
    def __setstate__(self, state): ...
    def __init__(self, d): ...
    def __len__(self): ...
    def __iter__(self): ...
    def __getitem__(self, key): ...
    def copy(self): ...
    def __str__(self): ...
    def __repr__(self): ...

class AdjacencyView(AtlasView):

    __slots__ = ...  # Still uses AtlasView slots names _atlas

    def __getitem__(self, name): ...
    def copy(self): ...

class MultiAdjacencyView(AdjacencyView):

    __slots__ = ...  # Still uses AtlasView slots names _atlas

    def __getitem__(self, name): ...
    def copy(self): ...

class UnionAtlas(Mapping):

    __slots__ = ...

    def __getstate__(self): ...
    def __setstate__(self, state): ...
    def __init__(self, succ, pred): ...
    def __len__(self): ...
    def __iter__(self): ...
    def __getitem__(self, key): ...
    def copy(self): ...
    def __str__(self): ...
    def __repr__(self): ...

class UnionAdjacency(Mapping):

    __slots__ = ...

    def __getstate__(self): ...
    def __setstate__(self, state): ...
    def __init__(self, succ, pred): ...
    def __len__(self): ...
    def __iter__(self): ...
    def __getitem__(self, nbr): ...
    def copy(self): ...
    def __str__(self): ...
    def __repr__(self): ...

class UnionMultiInner(UnionAtlas):

    __slots__ = ...  # Still uses UnionAtlas slots names _succ, _pred

    def __getitem__(self, node): ...
    def copy(self): ...

class UnionMultiAdjacency(UnionAdjacency):

    __slots__ = ...  # Still uses UnionAdjacency slots names _succ, _pred

    def __getitem__(self, node): ...

class FilterAtlas(Mapping):  # nodedict, nbrdict, keydict
    def __init__(self, d, NODE_OK): ...
    def __len__(self): ...
    def __iter__(self): ...
    def __getitem__(self, key): ...

    # FIXME should this just be removed? we don't use it, but someone might
    def copy(self): ...
    def __str__(self): ...
    def __repr__(self): ...

class FilterAdjacency(Mapping):  # edgedict
    def __init__(self, d, NODE_OK, EDGE_OK): ...
    def __len__(self): ...
    def __iter__(self): ...
    def __getitem__(self, node): ...

    # FIXME should this just be removed? we don't use it, but someone might
    def copy(self): ...
    def __str__(self): ...
    def __repr__(self): ...

class FilterMultiInner(FilterAdjacency):  # muliedge_seconddict
    def __iter__(self): ...
    def __getitem__(self, nbr): ...

    # FIXME should this just be removed? we don't use it, but someone might
    def copy(self): ...

class FilterMultiAdjacency(FilterAdjacency):  # multiedgedict
    def __getitem__(self, node): ...

    # FIXME should this just be removed? we don't use it, but someone might
    def copy(self): ...



from ._typing import *
from .path import Path

from collections import namedtuple
from functools import lru_cache

Page = ...
Box = ...

class Text(namedtuple("Text", "x y font glyph width")):
    @property
    def font_path(self) -> Path: ...
    @property
    def font_size(self): ...
    @property
    def font_effects(self): ...
    @property
    def glyph_name_or_index(self): ...

class Dvi:
    def __init__(self, filename, dpi: float) -> None: ...
    baseline = ...
    def __enter__(self): ...
    def __exit__(self, etype, evalue, etrace): ...
    def __iter__(self): ...
    def close(self) -> None: ...

class DviFont:
    def __init__(self, scale: float, tfm: Tfm, texname: bytes, vf: Vf) -> None: ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __repr__(self) -> str: ...

class Vf(Dvi):
    def __init__(self, filename: str | PathLike) -> None: ...
    def __getitem__(self, code): ...

class Tfm:
    def __init__(self, filename: str | PathLike) -> None: ...

PsFont = ...

class PsfontsMap:
    @lru_cache()
    def __new__(cls, filename): ...
    def __getitem__(self, texname): ...

class _LuatexKpsewhich:
    @lru_cache()
    def __new__(cls): ...
    def search(self, filename): ...

def find_tex_file(filename: str | PathLike, format: str | bytes = ...): ...
